/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.entities;

import fb.battlefield.Sprite;

/**
 * flying and shooting unit
 * 
 * @author farfpeuf
 * 
 */
public class FlyingShootingUnit extends Shootingunit implements FlyingEntity {
	/**
	 * delegate for flying methods
	 */
	private FlyingEntityDelegate flyingDelegate;

	/**
	 * create unit from line read in catalog
	 * 
	 * @param ligne
	 *            catalog line
	 * @param linenumber
	 *            line number
	 */
	public FlyingShootingUnit(String ligne, int linenumber) {
		super(ligne, linenumber);
		flyingDelegate = new FlyingEntityDelegate(this);

	}

	/**
	 * reset unit after end of battle
	 * 
	 * @param winner
	 *            did the unit army won the battle
	 */
	public void endBattle(boolean winner) {
		super.endBattle(winner);
		flyingDelegate.endBattle(winner);
	}

	/**
	 * @return sprite used when flying
	 */
	public Sprite getFlyingSprite() {
		return flyingDelegate.getFlyingSprite();
	}

	/**
	 * create entity sprites
	 * 
	 */
	public void createSprite() {
		super.createSprite();
		flyingDelegate.createsprite();
	}

	/**
	 * deletes sprite to free memory
	 * 
	 */
	public void freeSprite() {
		super.freeSprite();
		flyingDelegate.freeSprite();
	}

	/**
	 * @return shadow sprite used when flying
	 */
	public Sprite getShadowSprite() {
		return flyingDelegate.getShadowSprite();
	}

	/**
	 * returns unit as a string to save entity to a file
	 * 
	 * @return unit as a string
	 */
	public String saveString() {
		return super.saveString() + flyingDelegate.savestring();
	}

	/**
	 * load unit from a StringBuffer
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadString(StringBuffer buffy) {
		super.loadString(buffy);
		flyingDelegate.loadString(buffy);
	}

	/**
	 * is this entity currently flying
	 * 
	 * @return true if this entity currently flying
	 */
	public boolean isFlying() {
		return flyingDelegate.isFlying();
	}

	/**
	 * can this entity do a land manoeuvre
	 * 
	 * @return true if it can
	 */
	public boolean canLand() {
		return flyingDelegate.canLand();
	}

	/**
	 * can this entity do a take off manoeuvre
	 * 
	 * @return true if it can
	 */
	public boolean canTakeOff() {
		return flyingDelegate.canTakeOff();
	}

	/**
	 * do land manoeuvre
	 * 
	 */
	public void doLand() {
		flyingDelegate.doLand();
	}

	/**
	 * do take off manoeuvre
	 * 
	 */
	public void doTakeOff() {
		flyingDelegate.doTakeOff();
	}

	/**
	 * land manoeuvre
	 * 
	 */
	public void land() {
		flyingDelegate.land();
	}

	/**
	 * take off manoeuvre
	 * 
	 */
	public void takeOff() {
		flyingDelegate.takeOff();
	}

	/**
	 * clone this unit
	 */
	public Object clone() {
		FlyingShootingUnit unit = (FlyingShootingUnit) super.clone();
		unit.flyingDelegate = (FlyingEntityDelegate) flyingDelegate.clone(unit);
		return unit;
	}

	/**
	 * try to move entity to target entity
	 * 
	 * @param e
	 *            target entity
	 * @param ex
	 *            destination x coordinate
	 * @param ey
	 *            destination y coordinate
	 * @return true if a path was found
	 */
	public boolean moveTo(Entity e, int ex, int ey) {
		boolean result = super.moveTo(e, ex, ey);
		if (result) {
			flyingDelegate.takeOffLandTo(e);
		}
		return result;
	}
}
