/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.entities;

import fb.battlefieldelements.Sign;
import fb.util.Resources;

/**
 * a individual that can receive messages
 * 
 * @author farfpeuf
 * 
 */
public abstract class MessageReceiver extends Individual {

	/**
	 * new order received
	 */
	protected char newOrder = 'n';

	/**
	 * additional sign to display order
	 */
	private Sign ordersign;

	/**
	 * create inidividual from line read in catalog
	 * 
	 * @param ligne
	 *            catalog line
	 * @param linenumber
	 *            line number
	 */
	public MessageReceiver(String ligne, int linenumber) {

		super(ligne, linenumber);

	}

	/**
	 * initialize entity at the beginning of a battle
	 * 
	 */
	public void initBattle() {
		super.initBattle();
		ordersign = new Sign(this);
		signs.add(ordersign);
	}

	/**
	 * reset entity after end of battle
	 * 
	 * @param winner
	 *            did the entity army won the battle
	 */

	public void endBattle(boolean winner) {
		super.endBattle(winner);
		newOrder = 'n';
		ordersign = null;
	}

	/**
	 * change orders
	 * 
	 * @param neworder
	 *            new order
	 */
	public void setNewOrder(char neworder) {
		this.newOrder = neworder;
		if (neworder == 'a') {
			ordersign.setImage(Resources.getSign("attack"));
		} else if (neworder == 'o') {
			ordersign.setImage(Resources.getSign("oppose"));
		} else if (neworder == 'h') {
			ordersign.setImage(Resources.getSign("hold"));
		} else {
			ordersign.setImage(null);
		}
	}

	/**
	 * @return Returns the neworder.
	 */
	public char getNewOrder() {
		return newOrder;
	}

}
