/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.hierarchy;

import java.awt.Color;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.io.BufferedReader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;

import fb.army.entities.Entity;
import fb.army.entities.Exterminator;
import fb.army.entities.Hero;
import fb.army.entities.Leader;
import fb.army.entities.Legend;
import fb.main.Catalog;
import fb.main.FB;
import fb.main.FBConstants;
import fb.main.LoadingException;
import fb.phases.DeploymentPhase;
import fb.util.FBColors;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * an army
 * 
 * @author farfpeuf
 * 
 */
public class Army extends HierarchyNode {

	// property change constants

	public static final String POSITION = "position";

	// separator tag
	public static final String ARM_TAG = "#army#";

	/**
	 * icon for tree table
	 */
	private static Icon minicon;

	/**
	 * omens state
	 */
	private char omens = 'n';

	/**
	 * army characteristics
	 */
	private String playerName, race, list;

	/**
	 * army race id
	 */
	private int raceId;

	/**
	 * file path
	 */
	private String file;

	/**
	 * lists
	 */
	private Vector commands = new Vector(), allGroups = new Vector(),
			ennemies = new Vector(), friends = new Vector();

	/**
	 * active entities (neither dead no routed)
	 */
	private List activeEntities = new ArrayList();

	/**
	 * army leader
	 */
	private Leader warchief;

	/**
	 * must army make a morale test
	 */
	private boolean mustTestMorale = false;

	/**
	 * artificial intelligence or human player ?
	 */
	private boolean ai = false;

	/**
	 * handled by this computer ?
	 */
	private boolean handled = false;

	/**
	 * was army loaded from a file ?
	 */
	private boolean loaded = false;

	/**
	 * remaining move (campaign game)
	 */
	private int remainingMove = 0;

	/**
	 * id
	 */
	private int gmId = -1;

	/**
	 * side
	 */
	private int side;

	/**
	 * name
	 */
	private String name;

	/**
	 * force to wich it belongs
	 */
	private Force force;

	/**
	 * current position on screen
	 */
	private Point animationPosition = new Point();

	/**
	 * destination
	 */
	private Point position = new Point();

	/**
	 * movement path
	 */
	private LinkedList path = new LinkedList();

	/**
	 * campaign starting point
	 */
	private Point startPoint = new Point();

	/**
	 * constructor
	 * 
	 * @param race
	 *            the army race
	 * @param file
	 *            the file to load army from
	 */
	public Army(String race, String file) {
		setRace(race);
		this.file = file;
	}

	/**
	 * constructor
	 * 
	 * @param armylist
	 *            the army list
	 * @throws LoadingException
	 */
	public Army(String armylist) throws LoadingException {
		list = armylist;
		loaded = true;
		build();
		list = null;
	}

	/**
	 * is this army currently moving
	 * 
	 * @return true if it is
	 */
	public boolean isMoving() {
		return !position.equals(animationPosition);
	}

	/**
	 * constructor
	 * 
	 * @param playerName
	 *            player to which this army belongs
	 * @param armylist
	 *            the army list
	 * @param isia
	 *            is this army played by human or computer ?
	 * @param side
	 *            army side
	 * @param isHandled
	 *            is this army handled by this computer ?
	 */
	public Army(String playerName, String armylist, boolean isia, int side,
			boolean isHandled) {
		this.playerName = playerName;
		this.list = armylist;
		this.ai = isia;
		this.side = side;
		this.handled = isHandled;
	}

	/**
	 * destroy this army
	 * 
	 */
	public void destroy() {
		force.removeArmy(this);
	}

	/**
	 * merge this army with another army
	 * 
	 * @param other
	 *            the other army
	 */
	public void merge(Army other) {
		List otherGroups = other.getCommands();
		Command group;
		for (int i = 0; i < otherGroups.size(); i++) {
			group = (Command) otherGroups.get(i);
			if (group instanceof WarchiefCommand) {
				group = new Command(this, group);
			}
			addCommand(group);
		}
	}

	/**
	 * build this army from a file
	 * 
	 * @throws LoadingException
	 */
	public void build() throws LoadingException {
		StringBuffer ligne;
		String lineread;
		Command group = null;
		BufferedReader br;
		char ligneType;
		try {
			br = new BufferedReader(new StringReader(getList()));
			ligne = new StringBuffer(br.readLine());
			if (isLoaded()) {
				loadBattleString(ligne);
			} else {
				loadString(ligne);
			}
			lineread = br.readLine();
			if (lineread != null) {
				ligne = new StringBuffer(lineread);
			} else {
				ligne = null;
			}
			while (ligne != null) {
				ligneType = StringTools.readBufferChar(ligne);
				if (ligneType == 'w') {
					group = new WarchiefCommand();
					group.loadString(ligne);
					addCommand(group);
				} else if (ligneType == 'a') {
					group = new AllyCommand();
					group.loadString(ligne);
					addCommand(group);
				} else if (ligneType == 'g') {
					group = new Command();
					group.loadString(ligne);
					addCommand(group);
				} else if (ligneType == 'y') {
					group = new MercenaryCommand();
					group.loadString(ligne);
					addCommand(group);
				} else if (ligneType == 'e') {
					int linenumber = StringTools.readBufferInt(ligne);
					Entity entity = (Entity) ((Entity) Catalog.getFullCatalog()
							.get(linenumber)).clone();
					entity.loadString(ligne);
					entity.setLineNumber(linenumber);
					group.addEntity(entity);
				}
				lineread = br.readLine();
				if (lineread != null) {
					ligne = new StringBuffer(lineread);
				} else {
					ligne = null;
				}
			} // while
			br.close();
			this.list = null;
		} // try
		catch (Exception e) {
			throw new LoadingException(e);
		}
	}

	/**
	 * get icon corresponding to army race
	 * 
	 * @return icon corresponding to army race
	 */
	public ImageIcon getIcon() {
		return Resources.getSign(race);
	}

	/**
	 * reset army after end of battle
	 * 
	 * @param winner
	 *            did the army won the battle
	 */
	public void endBattle(boolean winner) {
		Command group;
		List oldGroups = new ArrayList(allGroups);
		activeEntities.clear();
		allGroups.clear();
		commands.clear();

		ennemies.clear();
		friends.clear();

		for (int i = 0; i < oldGroups.size(); i++) {
			group = (Command) oldGroups.get(i);
			group.setArmy(this);
			group.endBattle(winner);
			if (!group.isAnnihilated()
					&& !(group instanceof MercenaryCommand && ((MercenaryCommand) group)
							.hasBetrayed())) {
				addCommand(group);
				group.setOrder('o');
			}
		}
	}

	/**
	 * @return true if this army is annihilated
	 */
	public boolean isAnnihilated() {
		Command group;
		for (int i = 0; i < allGroups.size(); i++) {
			group = (Command) allGroups.get(i);
			if (!group.isAnnihilated()) {
				return false;
			}
		}
		return true;
	}

	/**
	 * calculate entities initial positions
	 * 
	 * @param rectangles
	 *            deployment area
	 * @param ennemyRectangles
	 *            ennemy deployment areas
	 */
	public void setInitialPosition(List rectangles, List ennemyRectangles) {

		Rectangle rect = (Rectangle) rectangles.get(0);
		startPoint = new Point((int) rect.getCenterX(), (int) rect.getCenterY());
		double initialAngle = 0.0;
		Area deployArea = new Area();
		Entity entity;
		int x = 0, y = 0;
		boolean placed;
		boolean changeRectangle = true;
		for (int i = 0; i < activeEntities.size(); i++) {
			entity = (Entity) activeEntities.get(i);
			if (!entity.isDeployed() && entity.isActive()) {
				placed = false;
				for (int j = 0; !placed && j < rectangles.size(); j++) {
					if (changeRectangle) {
						changeRectangle = false;
						rect = (Rectangle) rectangles.get(j);
						deployArea = new Area(rect);
						deployArea.subtract(entity.getMovementObstacles());
						initialAngle = DeploymentPhase.getInitialAngle(rect,
								ennemyRectangles);

						if (initialAngle == 0.0) {
							x = rect.x;
							y = rect.y;
						} else if (initialAngle == (Math.PI / 2)) {
							x = rect.x;
							y = rect.y + rect.height;
						} else if (initialAngle == Math.PI) {
							x = rect.x + rect.width;
							y = rect.y + rect.height;
						} else if (initialAngle == (3 * Math.PI / 2)) {
							x = rect.x + rect.width;
							y = rect.y;
						}

					}

					entity.doRotate(initialAngle);

					while (!placed && !changeRectangle) {
						// System.out.println(x+" "+y+" for
						// "+entity.getfullname());
						if (entity.canDeployTo(x, y, deployArea)) {
							entity.doDeployTo(x, y);
							deployArea.subtract(entity.getArea());
							if (initialAngle == 0.0) {
								x += (entity.getFrontWidth() / 2);
							} else if (initialAngle == (Math.PI / 2)) {
								y -= (entity.getFrontWidth() / 2);
							} else if (initialAngle == Math.PI) {
								x -= (entity.getFrontWidth() / 2);
							} else if (initialAngle == (3 * Math.PI / 2)) {
								y += (entity.getFrontWidth() / 2);
							}

							placed = true;
						}
						if (initialAngle == 0.0) {
							x += (entity.getWidth() * 8);
							if (x > rect.x + rect.width) {
								x = rect.x;
								y += (entity.getLength() * 8);
								if (y > rect.y + rect.height) {
									changeRectangle = true;
								}
							}
						} else if (initialAngle == (Math.PI / 2)) {
							y -= (entity.getWidth() * 8);
							if (y < rect.y) {
								y = rect.y + rect.height;
								x += (entity.getLength() * 8);
								if (x > (rect.x + rect.width)) {
									changeRectangle = true;
								}
							}
						} else if (initialAngle == Math.PI) {
							x -= (entity.getWidth() * 8);
							if (x < rect.x) {
								x = rect.x + rect.width;
								y -= (entity.getLength() * 8);
								if (y < rect.y) {
									changeRectangle = true;
								}
							}
						} else if (initialAngle == (3 * Math.PI / 2)) {
							y += (entity.getWidth() * 8);
							if (y > (rect.y + rect.height)) {
								y = rect.y;
								x -= (entity.getLength() * 8);
								if (x < rect.x) {
									changeRectangle = true;
								}
							}
						}

					}

				}

			}
		}
	}

	/**
	 * load entities initial positions
	 * 
	 */
	public void loadInitialPosition() {
		Entity entity;
		int x, y;
		for (int i = 0; i < activeEntities.size(); i++) {
			entity = (Entity) activeEntities.get(i);
			entity.doRotate(entity.getInitialAngle());
			x = entity.getInitialX();
			y = entity.getInitialY();
			entity.doDeployTo(x, y);

		}
	}

	/**
	 * calculate cost
	 */
	public int getCost() {
		int cost = 0;
		for (int i = 0; i < commands.size(); i++) {
			cost += ((Command) commands.get(i)).getCost();
		}
		return cost;
	}

	/**
	 * number of warriors
	 */
	public int getNumberOfWarriors() {
		int number = 0;
		for (int i = 0; i < commands.size(); i++) {
			number += ((Command) commands.get(i)).getNumberOfWarriors();
		}
		return number;
	}

	/**
	 * @return maximum movement
	 */
	public int movement() {
		if (commands.size() == 0) {
			return 0;
		}
		int movement = Integer.MAX_VALUE;
		for (int i = 0; i < commands.size(); i++) {
			movement = Math.min(movement, ((Command) commands.get(i))
					.movement());
		}
		return Math.max(3, movement);
	}

	/**
	 * calculate army degradation
	 * 
	 * @return army degradation
	 */
	public int degradation() {
		int degradation = 0;
		Entity entity;
		for (int i = 0; i < activeEntities.size(); i++) {
			entity = (Entity) activeEntities.get(i);
			if (entity.getState() == 'r' || entity.getState() == 'a') {
				degradation += 1;
			}
		}
		return degradation;
	}

	/**
	 * test army morale
	 * 
	 */
	public void moraleTest() {
		for (int i = 0; i < commands.size(); i++) {
			((Command) commands.get(i)).moraleTest();
		}
		mustTestMorale = false;
	}

	/**
	 * rout this army
	 * 
	 */
	public void rout() {
		for (int i = 0; i < commands.size(); i++) {
			((Command) commands.get(i)).groupReaction(0);
		}
	}

	/**
	 * calculate routed and annihilated commands
	 * 
	 * @return number of routed and annihilated commands
	 */
	public int routedAnnihilatedCommands() {
		int result = 0;
		for (int i = 0; i < commands.size(); i++) {
			if (((Command) commands.get(i)).isRoutedAnnihilated()) {
				result++;
			}
		}
		return result;
	}

	/**
	 * add a ennemy to this army
	 * 
	 * @param ennemy
	 *            the ennemy army
	 */
	public void addEnnemy(Army ennemy) {
		ennemies.add(ennemy);
	}

	/**
	 * add a friendly army to this army
	 * 
	 * @param ennemy
	 *            the friendly army
	 */
	public void addFriend(Army friend) {
		friends.add(friend);
	}

	/**
	 * recruit a new command
	 * 
	 * @param command
	 *            the new command
	 */
	public void recruitCommand(Command command) {
		allGroups.add(command);
		commands.add(command);
		command.army = this;
	}

	/**
	 * fire a command
	 * 
	 * @param command
	 *            the command to fire
	 */
	public void fireCommand(Command command) {
		allGroups.remove(command);
		commands.remove(command);
		command.army = null;
	}

	/**
	 * add a new command
	 * 
	 * @param command
	 *            the new command
	 */
	public void addCommand(Command command) {
		allGroups.add(command);
		commands.add(command);
		command.army = this;
	}

	/**
	 * remove a command
	 * 
	 * @param command
	 *            the command to remove
	 */
	public void removeCommand(Command command) {
		commands.remove(command);
		command.army = null;
	}

	/**
	 * add a new entity
	 * 
	 * @param e
	 *            the new entity
	 */
	public void addEntity(Entity e) {
		activeEntities.add(e);
	}

	/**
	 * remove an entity
	 * 
	 * @param command
	 *            the entity to remove
	 */
	public void removeEntity(Entity e) {
		activeEntities.remove(e);
	}

	/**
	 * Sets the race.
	 * 
	 * @param race
	 *            The race to set
	 */
	public void setRace(String race) {
		this.race = race;
		if (race.equals("barbarian")) {
			raceId = FBConstants.BARBARIAN;
		} else if (race.equals("dwarf")) {
			raceId = FBConstants.DWARF;
		} else if (race.equals("goblin")) {
			raceId = FBConstants.GOBLIN;
		} else if (race.equals("high-elf")) {
			raceId = FBConstants.HIGHELF;
		} else if (race.equals("human")) {
			raceId = FBConstants.HUMAN;
		} else if (race.equals("woodelf")) {
			raceId = FBConstants.WOODELF;
		} else if (race.equals("orc")) {
			raceId = FBConstants.ORC;
		}
	}

	/**
	 * Returns the gmId.
	 * 
	 * @return int
	 */
	public int getGmId() {
		return gmId;
	}

	/**
	 * get index in force army list
	 * 
	 * @return index in force army list
	 */
	public int getForceIndex() {
		return force.getIndex(this);
	}

	/**
	 * Sets the gmId.
	 * 
	 * @param gmId
	 *            The gmId to set
	 */
	public void setGmId(int id) {
		this.gmId = id;
	}

	/**
	 * returns army as a string to save it to a file
	 * 
	 * @return army as a string
	 */
	public String saveArmyString() {
		String savestring = race + '\t' + '\n';
		for (int i = 0; i < allGroups.size(); i++) {
			savestring += (((Command) allGroups.get(i)).saveString());
		} // for
		return savestring;
	}

	/**
	 * load army from a StringBuffer
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadString(StringBuffer buffy) {
		setRace(StringTools.readBuffer(buffy));
	}

	/**
	 * returns army as a string to save it to a file when saving a battle
	 * 
	 * @return army as a string
	 */
	public String saveBattleString() {
		String savestring = "" + position.x + '\t' + position.y + '\t' + race
				+ '\t' + playerName + '\t' + gmId + '\t' + startPoint.x + ','
				+ startPoint.y + '\t' + side + '\t' + (ai ? 'y' : 'n') + '\t'
				+ (handled ? 'y' : 'n') + '\t' + (mustTestMorale ? 'y' : 'n')
				+ '\t' + remainingMove + '\n';
		for (int i = 0; i < allGroups.size(); i++) {
			savestring += (((Command) allGroups.get(i)).saveString());
		} // for
		return savestring;
	}

	/**
	 * load army from a StringBuffer when loading a battle
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadBattleString(StringBuffer buffy) {
		Point p = new Point();
		p.x = StringTools.readBufferInt(buffy);
		p.y = StringTools.readBufferInt(buffy);

		initPosition(p);

		setRace(StringTools.readBuffer(buffy));
		playerName = StringTools.readBuffer(buffy);
		gmId = StringTools.readBufferInt(buffy);
		String startPointString = StringTools.readBuffer(buffy);
		String[] coords = startPointString.split(",");

		if (coords.length > 1) {
			startPoint = new Point(Integer.parseInt(coords[0]), Integer
					.parseInt(coords[1]));
		} else {
			startPoint = new Point();
		}

		side = StringTools.readBufferInt(buffy);
		ai = (StringTools.readBufferChar(buffy) == 'y') ? true : false;
		handled = (StringTools.readBufferChar(buffy) == 'y') ? true : false;
		mustTestMorale = (StringTools.readBufferChar(buffy) == 'y') ? true
				: false;
		remainingMove = StringTools.readBufferInt(buffy);
	}

	/**
	 * @see javax.swing.tree.TreeNode#children()
	 */
	public Enumeration children() {
		return commands.elements();
	}

	/**
	 * @see javax.swing.tree.TreeNode#getAllowsChildren()
	 */
	public boolean getAllowsChildren() {
		return true;
	}

	/**
	 * @see javax.swing.tree.TreeNode#getChildAt(int)
	 */
	public TreeNode getChildAt(int childIndex) {
		return (TreeNode) commands.get(childIndex);
	}

	/**
	 * @see javax.swing.tree.TreeNode#getChildCount()
	 */
	public int getChildCount() {
		return commands.size();
	}

	/**
	 * @see javax.swing.tree.TreeNode#getIndex(TreeNode)
	 */
	public int getIndex(TreeNode node) {
		return commands.indexOf(node);
	}

	/**
	 * @see javax.swing.tree.TreeNode#getParent()
	 */
	public TreeNode getParent() {
		return force;
	}

	/**
	 * @see javax.swing.tree.TreeNode#isLeaf()
	 */
	public boolean isLeaf() {
		return false;
	}

	/**
	 * Returns the name.
	 * 
	 * @return String
	 */
	public String getName() {
		if (name == null) {
			name = Resources.getString("theracearmy").replaceAll("<race>",
					Resources.getString("ae" + race).toLowerCase());
		}
		return name;
	}

	/**
	 * @see javax.swing.tree.MutableTreeNode#insert(MutableTreeNode, int)
	 */
	public void insert(MutableTreeNode child, int index) {
		recruitCommand((Command) child);
	}

	/**
	 * @see javax.swing.tree.MutableTreeNode#remove(int)
	 */
	public void remove(int index) {
		fireCommand((Command) commands.get(index));
	}

	/**
	 * @see javax.swing.tree.MutableTreeNode#remove(MutableTreeNode)
	 */
	public void remove(MutableTreeNode node) {
		fireCommand((Command) node);
	}

	/**
	 * @see javax.swing.tree.MutableTreeNode#removeFromParent()
	 */
	public void removeFromParent() {
		force.remove(this);
	}

	/**
	 * @see javax.swing.tree.MutableTreeNode#setParent(MutableTreeNode)
	 */
	public void setParent(MutableTreeNode newParent) {
		setForce((Force) newParent);
	}

	/**
	 * @see javax.swing.tree.MutableTreeNode#setUserObject(Object)
	 */
	public void setUserObject(Object object) {
	}

	/**
	 * @return army full name
	 */
	public String getFullName() {
		return getName();
	}

	/**
	 * @return icon for tree table
	 */

	public Icon getMinicon() {
		if (null == minicon) {
			minicon = new ImageIcon((Resources.getSign("army")).getImage()
					.getScaledInstance(20, 20, Image.SCALE_SMOOTH));
		}
		return minicon;
	}

	/**
	 * @return true if this army is route or annihilated
	 */
	public boolean isRoutedAnnihilated() {
		Iterator iter = commands.iterator();
		while (iter.hasNext()) {
			if (!((Command) iter.next()).isRoutedAnnihilated()) {
				return false;
			}
		}
		return true;
	}

	/**
	 * return number of entities of specified class
	 * 
	 * @param cl
	 *            class to search for
	 * @return number of entities of specified class
	 */
	public int numberOf(Class cl) {
		int number = 0;
		Iterator iter = allGroups.iterator();
		while (iter.hasNext()) {
			number += ((Command) iter.next()).numberOf(cl);
		}
		return number;
	}

	/**
	 * check army content before saving
	 * 
	 * @param budget
	 *            maximum budget
	 * @return null if army is ok or an error message
	 */
	public String check(int budget) {
		String message = "";
		if (FB.getInstance().isDebug()) {
			return message;
		}
		if (getCost() > budget) {
			message += Resources.getString("moreThanBudget");
		}
		Iterator iter = allGroups.iterator();
		int nonStrangers = 0, strangers = 0;
		Command group;
		while (iter.hasNext()) {
			group = (Command) iter.next();
			message += (group).check();
			if (group instanceof StrangerCommand) {
				strangers += group.numberOfModels();
			} else {
				nonStrangers += group.numberOfModels();
			}
		}
		if (strangers >= nonStrangers) {
			message += Resources.getString("moreStrangers");
		}
		if (numberOf((new Hero(null, 0)).getClass()) > 1) {
			message += Resources.getString("moreThanOneHero");
		}
		if (numberOf((new Legend(null, 0)).getClass()) > 1) {
			message += Resources.getString("moreThanOnePaladin");
		}
		if (numberOf((new Exterminator(null, 0)).getClass()) > 1) {
			message += Resources.getString("moreThanOneDestroyer");
		}

		return message;
	}

	/**
	 * number of stranger warriors in this army
	 * 
	 * @return number of stranger warriors in this army
	 */
	public int nbStrangers() {

		Iterator iter = allGroups.iterator();
		int strangers = 0;
		Command group;
		while (iter.hasNext()) {
			group = (Command) iter.next();
			if (group instanceof StrangerCommand) {
				strangers += group.numberOfModels();
			}
		}
		return strangers;

	}

	/**
	 * number of non stranger warriors in this army
	 * 
	 * @return number of non stranger warriors in this army
	 */
	public int nbNonStrangers() {

		Iterator iter = allGroups.iterator();
		int nonStrangers = 0;
		Command group;
		while (iter.hasNext()) {
			group = (Command) iter.next();
			if (!(group instanceof StrangerCommand)) {
				nonStrangers += group.numberOfModels();
			}
		}
		return nonStrangers;

	}

	/**
	 * @return army start point
	 */
	public Point getStartPoint() {
		return startPoint;
	}

	/**
	 * @return Returns the allGroups.
	 */
	public Vector getAllGroups() {
		return allGroups;
	}

	/**
	 * @return Returns the force.
	 */
	public Force getForce() {
		return force;
	}

	/**
	 * @param startPoint
	 *            The startPoint to set.
	 */
	public void setStartPoint(Point startPoint) {
		this.startPoint = startPoint;
	}

	/**
	 * get army color
	 * 
	 * @return army color
	 */
	public Color getColor() {
		if (force != null) {
			return force.getColor();
		} else {
			return FBColors.color[gmId];
		}
	}

	/**
	 * set army force
	 * 
	 * @param force
	 *            army force
	 */
	public void setForce(Force force) {
		this.force = force;
		if (force != null) {
			this.playerName = force.getPlayerName();
			this.side = force.getSide();
			this.ai = force.isAi();
			this.handled = force.isHandled();
		}
	}

	/**
	 * 
	 * move army to another destination
	 * 
	 * @param path
	 *            movement path
	 * @param cost
	 *            movement cost
	 */
	public void move(LinkedList path, int cost) {
		if (!path.isEmpty()) {
			Point destination = (Point) path.getLast();
			remainingMove -= cost;
			this.position = destination;
			this.path = path;
		}
	}

	/**
	 * animate army movement
	 * 
	 * @return true if army moved
	 */
	public boolean animate() {
		if (!position.equals(animationPosition)) {
			setAnimationPosition((Point) path.removeFirst());
			return true;
		}
		return false;
	}

	/**
	 * initialize army position
	 * 
	 * @param hexaCoord
	 *            army initial position
	 */
	public void initPosition(Point hexaCoord) {
		this.animationPosition = hexaCoord;
		this.position = hexaCoord;
	}

	/**
	 * change army position
	 * 
	 * @param hexaCoord
	 *            new army position
	 */
	public void setAnimationPosition(Point hexaCoord) {
		Point oldCoord = this.animationPosition;
		this.animationPosition = hexaCoord;
		firePropertyChange(POSITION, oldCoord, hexaCoord);
	}

	/**
	 * @return Returns the animationPosition.
	 */
	public Point getAnimationPosition() {
		return animationPosition;
	}

	/**
	 * @return Returns the loaded.
	 */
	public boolean isLoaded() {
		return loaded;
	}

	/**
	 * @return Returns the side.
	 */
	public int getSide() {
		return side;
	}

	/**
	 * @return Returns the mustTestMorale.
	 */
	public boolean mustTestMorale() {
		return mustTestMorale;
	}

	/**
	 * @param mustTestMorale
	 *            The mustTestMorale to set.
	 */
	public void setMustTestMorale(boolean mustTestMorale) {
		this.mustTestMorale = mustTestMorale;
	}

	/**
	 * @return Returns the omens.
	 */
	public char getOmens() {
		return omens;
	}

	/**
	 * @param omens
	 *            The omens to set.
	 */
	public void setOmens(char omens) {
		this.omens = omens;
	}

	/**
	 * @return Returns the warchief.
	 */
	public Leader getWarchief() {
		return warchief;
	}

	/**
	 * @param warchief
	 *            The warchief to set.
	 */
	public void setWarchief(Leader warchief) {
		this.warchief = warchief;
	}

	/**
	 * @return Returns the ai.
	 */
	public boolean isAi() {
		return ai;
	}

	/**
	 * @return Returns the handled.
	 */
	public boolean isHandled() {
		return handled;
	}

	/**
	 * @return Returns the activeEntities.
	 */
	public List getActiveEntities() {
		return activeEntities;
	}

	/**
	 * @return Returns the file.
	 */
	public String getFile() {
		return file;
	}

	/**
	 * @return Returns the ennemies.
	 */
	public Vector getEnnemies() {
		return ennemies;
	}

	/**
	 * @return Returns the friends.
	 */
	public Vector getFriends() {
		return friends;
	}

	/**
	 * @return Returns the groups.
	 */
	public Vector getCommands() {
		return commands;
	}

	/**
	 * @return Returns the list.
	 */
	public String getList() {
		return list;
	}

	/**
	 * @return Returns the race.
	 */
	public String getRace() {
		return race;
	}

	/**
	 * @return Returns the remainingMove.
	 */
	public int getRemainingMove() {
		return remainingMove;
	}

	/**
	 * reset army remaining movement points
	 * 
	 */
	public void resetRemainingMove() {
		this.remainingMove = movement();
	}

	/**
	 * clear army remaining movement points
	 * 
	 */
	public void noRemainingMove() {
		this.remainingMove = 0;
	}

	/**
	 * @return true if this army has a mercenary command
	 */
	public boolean hasMercenary() {
		Command group;
		Iterator iter = getAllGroups().iterator();
		while (iter.hasNext()) {
			group = (Command) iter.next();
			if (group instanceof MercenaryCommand) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return true if this army has an allied command
	 */
	public boolean hasAllied() {
		Command group;
		Iterator iter = getAllGroups().iterator();
		while (iter.hasNext()) {
			group = (Command) iter.next();
			if (group instanceof AllyCommand) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return Returns the position.
	 */
	public Point getPosition() {
		return position;
	}

	/**
	 * @return Returns the playerName.
	 */
	public String getPlayerName() {
		return playerName;
	}

	/**
	 * @return Returns the raceId.
	 */
	public int getRaceId() {
		return raceId;
	}

}