/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.hierarchy;

import fb.army.entities.Entity;
import fb.main.FB;
import fb.network.Client;
import fb.network.MessagesConstants;
import fb.util.D100;
import fb.util.DX;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * a mercenary command
 * 
 * @author farfpeuf
 * 
 */
public class MercenaryCommand extends StrangerCommand {

	/**
	 * has this command already betrayed its army
	 */
	private boolean hasBetrayed = false;

	/**
	 * returns command as a string to save it to a file
	 * 
	 * @return command as a string
	 */
	public String saveString() {
		String savestring = "y" + '\t' + name + '\t' + order + '\t'
				+ (changedOrders ? 'y' : 'n') + '\t' + 'p' + '\t'
				+ (hasBetrayed ? 'y' : 'n') + '\n';
		for (int i = 0; i < allEntities.size(); i++) {
			savestring += (((Entity) allEntities.get(i)).saveString() + '\n');
		} // for
		return savestring;
	}

	/**
	 * load command from a StringBuffer
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadString(StringBuffer buffy) {
		super.loadString(buffy);
		StringTools.readBufferChar(buffy);
		hasBetrayed = (StringTools.readBufferChar(buffy) == 'y') ? true : false;
	}

	/**
	 * test command morale
	 * 
	 */
	public void moraleTest() {
		int score = D100.roll();

		if (leader != null && leader.getState() == 'n') {
			score += (leader.getLeadership() * 15);
		}

		if (army.getWarchief() != null || army.getWarchief().getState() == 'r') {
			score -= 45;
		}

		if (army.getOmens() == 'b') {
			score -= 15;
		}
		if (suffersBadLight()) {
			score -= 15;
		}
		if (army.getOmens() == 'g') {
			score += 15;
		}
		score -= (army.routedAnnihilatedCommands() * 15);

		score -= 15;

		if (score < 65) {
			betrayalTest();
		} else {
			groupReaction(score);
		}
	}

	/**
	 * do caculate reaction to morale test
	 * 
	 * @param score
	 *            morale test score
	 */
	public void doGroupReaction(int score) {

		if (score >= 95) {
			setBloodlust();
		} else if (score >= 65) {
		} else if (score >= 50) {
			setShaken();
		} else {
			rout();
		}

	}

	/**
	 * test if command betrays its army
	 * 
	 */
	public void betrayalTest() {
		int score = D100.roll();

		if (score >= 50 || hasBetrayed) {
			score -= 30;
			groupReaction(score);
		} else {
			Army ennemyarmy = (Army) army.getEnnemies().get(
					DX.roll(army.getEnnemies().size()) - 1);

			betray(ennemyarmy);
		}

	}

	/**
	 * betray army
	 * 
	 * @param ennemyArmy
	 *            the new army
	 */
	public void betray(Army ennemyArmy) {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(
					MessagesConstants.BETRAY + army.getGmId() + "#" + getArmIndex()
							+ "#" + ennemyArmy.getGmId());
		} else {
			doBetray(ennemyArmy);
		}
	}

	/**
	 * do betray army
	 * 
	 * @param ennemyArmy
	 *            the new army
	 */
	public void doBetray(Army ennemyArmy) {

		hasBetrayed = true;
		String message = Resources.getString("betrayal");
		message = StringTools.replaceAll(message, "<group name>", name);
		FB.getInstance().message(message, army.getColor());
		FB.getInstance().popUpMessage(Resources.getString("betrayalTittle"),
				message);

		Entity e;

		for (int i = 0; i < activeEntities.size(); i++) {
			e = (Entity) activeEntities.get(i);
			e.betray(ennemyArmy);
		}
		army.fireCommand(this);
		ennemyArmy.addCommand(this);

	}

	/**
	 * @return ture if this command has already betrayed its army
	 */
	public boolean hasBetrayed() {
		return hasBetrayed;
	}

	/**
	 * @return list of races that can be added to this command
	 */
	public String[] getAvailableRaces() {
		return mercenaries;
	}

	/**
	 * list of races that can be added to a mercenary command
	 */
	private static final String[] mercenaries = new String[] {
	// RB.getString("aeamazon"),
			Resources.getString("aebarbarian"),
			Resources.getString("aehugecreature"),
			Resources.getString("aetroll"), Resources.getString("aeogre"),
			Resources.getString("aegoblin"), Resources.getString("aedwarf"),
			Resources.getString("aewoodelf"), Resources.getString("aehuman"),
			Resources.getString("aeorc"), Resources.getString("aehigh-elf") };

}
