/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.battlefield;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.io.File;
import java.util.Enumeration;
import java.util.List;

import javax.swing.ImageIcon;

import fb.battlefieldelements.VerticalSet;
import fb.main.FB;
import fb.util.FBColors;
import fb.util.FBImage;
import fb.util.Resources;
/**
 * delegate class used to draw on a mini-battlefield map
 * @author farfpeuf
 *
 */
public class MiniMapPainter {
	/**
	 * draw all the map elements
	 * @param g graphics
	 * @param mm the map data model
	 * @param scale the scale
	 * @param battleMap true if the map is  used for battle (false means map is used for game preparation)
	 */
	private static void drawAll(
		Graphics g,
		MapModel mm,
		float scale,
		boolean battleMap) {
		drawVSets(g, mm, scale);
		if (!battleMap) {
			drawSet(g, mm, 'a', scale);
			g.setColor(FBColors.color2[0]);
			drawSet(g, mm, 'b', scale);
			g.setColor(FBColors.color2[1]);
			drawSet(g, mm, 'c', scale);
			g.setColor(FBColors.color2[2]);
			drawSet(g, mm, 'd', scale);
			g.setColor(FBColors.color2[3]);
			drawSet(g, mm, 'e', scale);
			g.setColor(FBColors.color2[4]);
			drawSet(g, mm, 'f', scale);
		}
	}
	
	/**
	 * draw vertical sets (trees and buildings) on the map
	 * @param g graphics
	 * @param mm the map data model
	 * @param scale the scale
	 */
	private static void drawVSets(Graphics g, MapModel mm, float scale) {
		Enumeration keys = mm.getVerticalSets().keys();
		String key;
		List vset;
		FBImage image;
		Point p;
		while (keys.hasMoreElements()) {
			key = (String) keys.nextElement();
			vset = (List) mm.getVerticalSets().get(key);
			image = VerticalSet.getFantasyImage(key);
			for (int i = 0; i < vset.size(); i++) {
				p = (Point) vset.get(i);
				image.draw(g, (int) p.getX(), (int) p.getY(), scale, 0, 0);
			}
		}
	}
	
	/**
	 * draw a set of shapes
	 * @param g graphics
	 * @param mm the map data model
	 * @param type the type of set
	 * @param scale the scale
	 */
	private static void drawSet(
		Graphics g,
		MapModel mm,
		char type,
		float scale) {
		List set = mm.getSet(type);
		if (set != null) {
			for (int i = 0; i < set.size(); i++) {
				drawRectangle(
					g,
					(Rectangle) set.get(i),
					String.valueOf(type).toUpperCase(),
					scale);
			}
		}
	}
	
	/**
	 * draw a rectagle on the map
	 * @param g graphics
	 * @param rect the rectangle
	 * @param name the label to write inside the rectangle
	 * @param scale the scale
	 */
	private static void drawRectangle(
		Graphics g,
		Rectangle rect,
		String name,
		float scale) {

		FontMetrics metrics =
			Toolkit.getDefaultToolkit().getFontMetrics(g.getFont());
		g.drawRect(
			(int) (rect.x * scale),
			(int) (rect.y * scale),
			(int) (rect.width * scale),
			(int) (rect.height * scale));
		g.drawString(
			name,
			(int) (rect.getCenterX() * scale),
			(int) (rect.getCenterY() * scale + metrics.getAscent() / 2));
	}
	
	/**
	 * get map as an image icon
	 * @param name map name
	 * @param battleMap true if the map is  used for battle (false means map is used for game preparation)
	 * @return map as an image icon
	 */
	public static ImageIcon getPaintedMap(String name, boolean battleMap) {
		FB.getInstance().setFrameCursor(FB.DEFAULT_WAIT_CURSOR);

		MapModel.getInstance().load(
			"mapmodels" + File.separator + name);
		int MAX_WIDTH, MAX_LENGTH;
		if (battleMap) {
			MAX_WIDTH = 144;
			MAX_LENGTH = 96;
		} else {
			MAX_WIDTH = 192;
			MAX_LENGTH = 128;
		}

		float scaleW = MAX_WIDTH / (float) MapModel.getInstance().getMapWidth();
		float scaleL =
			MAX_LENGTH / (float) MapModel.getInstance().getMapLength();
		float scale;
		if (scaleW < scaleL) {
			scale = scaleW;
		} else {
			scale = scaleL;
		}
		int width = (int) (MapModel.getInstance().getMapWidth() * scale);
		int length = (int) (MapModel.getInstance().getMapLength() * scale);
		Image bgImg =
			Resources.getMap(
				MapModel.getInstance().getImageName(),
				MapModel.getInstance().isFlipLR(),
				MapModel.getInstance().isFlipUD());
		MediaTracker tracker = new MediaTracker(FB.getInstance());
		tracker.addImage(bgImg, 0);
		try {
			tracker.waitForAll();
		} catch (InterruptedException e) {
			FB.exception(e);
		}
		Image paintedImage = FB.getInstance().createImage(width, length);
		Graphics graf = paintedImage.getGraphics();
		graf.drawImage(bgImg, 0, 0, width, length, null);
		drawAll(graf, MapModel.getInstance(), scale, battleMap);
		FB.getInstance().setDefaultCursor();
		return new ImageIcon(paintedImage);
	}

}