/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.battlemapeditor;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.EventObject;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextField;

import fb.battlefield.MapModel;
import fb.battlefieldelements.VerticalSet;
import fb.gui.frames.popups.FilePopup;
import fb.gui.widgets.ButtonListener;
import fb.main.Options;
import fb.util.FBColors;
import fb.util.FileTools;
import fb.util.PercentageLayout;
import fb.util.Resources;
/**
 * a tool to create and edit battle map models
 * @author farfpeuf
 *
 */
public class MapModelEditor extends JFrame {
	/**
	 * the frame content pane
	 */
	public Container contentPane;
	/**
	 * the map panel
	 */
	public MapPanel map;
/**
 * the graphics device
 */
	public static GraphicsDevice gd;
/**
 * the desktop pane containing the map panel
 */
	public JDesktopPane mapPanel;
	/**
	 * the current obstacle/set type
	 */
	public JLabel currentTypeLabel;
	/**
	 * menus
	 */
	public JMenu mainmenu, typemenu, vsetMenu, sideMenu;
	/**
	 * sub menus
	 */
	public JMenuItem exit,
		newmap,
		openimage,
		openmap,
		savemap,
		saveasmap,
		river,
		wood,
		marais,
		fog,
		house,
		aSide,
		bSide,
		cSide,
		dSide,
		eSide,
		fSide;
	/**
	 * vertical sets sub menus
	 */
	public JMenuItem[] vsets;
	/**
	 * undo button
	 */
	public JButton undoLast;
	/**
	 * size text fields
	 */
	public JTextField width, length;

/**
 * programm main method
 * @param args command line arguments
 */
	public static void main(String[] args) {

		MapModelEditor editor = new MapModelEditor();

		gd =
			GraphicsEnvironment
				.getLocalGraphicsEnvironment()
				.getDefaultScreenDevice();

		gd.setFullScreenWindow(editor);

	}
	/**
	 * constructor
	 * build frame and register event
	 */
	public MapModelEditor() {

		Resources.setResourceBundle(
			ResourceBundle.getBundle(
				"Messages",
				new Locale(
					Options.getLanguage(),
					Options.getCountry())));
		setBackground(FBColors.background);

		contentPane = getContentPane();
		contentPane.setBackground(FBColors.background);
		buildFrame();

	}
	/**
	 * build and assemble the components
	 *
	 */
	private void buildFrame() {

		map = new MapPanel(MapModel.getInstance());
		contentPane.removeAll();
		contentPane.repaint();
		JLabel fond = new JLabel();
		currentTypeLabel = new JLabel();
		JMenuBar menubar; //barre de menu
		width = new JTextField("576");
		length = new JTextField("384");
		//construire la barre des menus
		menubar = new JMenuBar();
		mainmenu = new JMenu(Resources.getString("Map"));
		newmap = new JMenuItem(Resources.getString("Newmap"));
		openmap = new JMenuItem(Resources.getString("Openmap"));
		openimage = new JMenuItem(Resources.getString("Openimage"));
		undoLast = new JButton(Resources.getString("RemoveLast"));
		savemap = new JMenuItem(Resources.getString("Save"));
		saveasmap = new JMenuItem(Resources.getString("Saveas"));
		exit = new JMenuItem(Resources.getString("Exit"));
		mainmenu.add(newmap);
		mainmenu.add(openmap);
		mainmenu.add(openimage);
		mainmenu.add(savemap);
		mainmenu.add(saveasmap);
		mainmenu.add(exit);
		typemenu = new JMenu(Resources.getString("Obstacletype"));
		river = new JMenuItem(Resources.getString("Riverobstacle"));
		wood = new JMenuItem(Resources.getString("Woodobstacle"));
		marais = new JMenuItem(Resources.getString("Maraisobstacle"));
		fog = new JMenuItem(Resources.getString("Fogobstacle"));
		house = new JMenuItem(Resources.getString("Houseobstacle"));
		typemenu.add(river);
		typemenu.add(wood);
		typemenu.add(marais);
		typemenu.add(fog);
		typemenu.add(house);
		vsetMenu = new JMenu(Resources.getString("Verticalsets"));
		JMenu vsetSubMenu = null;
		List vsetList = FileTools.list("vset");
		if (!vsetList.isEmpty()) {

			char vsetChar;
			JMenuItem item;
			Icon icon;
			String name;
			vsets = new JMenuItem[vsetList.size()];
			for (int i = 0; i < vsets.length; i++) {
				if (i % 5 == 0) {
					vsetSubMenu = new JMenu("" + (i + 1) + " - " + (i + 6));
					vsetMenu.add(vsetSubMenu);
				}
				vsetChar = ((String)vsetList.get(i)).charAt(0);
				name = "" + vsetChar;
				icon =
					new ImageIcon(
						VerticalSet
							.getFantasyImage(name)
							.getImage()
							.getScaledInstance(
							32,
							32,
							Image.SCALE_FAST));
				item = new JMenuItem("" + vsetChar, icon);
				vsets[i] = item;
				vsetSubMenu.add(item);
			}
		}
		sideMenu = new JMenu(Resources.getString("Side"));
		aSide = new JMenuItem(Resources.getString("Side") + " A");
		bSide = new JMenuItem(Resources.getString("Side") + " B");
		cSide = new JMenuItem(Resources.getString("Side") + " C");
		dSide = new JMenuItem(Resources.getString("Side") + " D");
		eSide = new JMenuItem(Resources.getString("Side") + " E");
		fSide = new JMenuItem(Resources.getString("Side") + " F");
		sideMenu.add(aSide);
		sideMenu.add(bSide);
		sideMenu.add(cSide);
		sideMenu.add(dSide);
		sideMenu.add(eSide);
		sideMenu.add(fSide);
		menubar.add(mainmenu);
		menubar.add(typemenu);
		menubar.add(vsetMenu);
		menubar.add(sideMenu);
		JPanel toolBar = new JPanel();
		toolBar.setLayout(new FlowLayout());
		toolBar.add(currentTypeLabel);
		toolBar.add(undoLast);
		toolBar.add(new JLabel(Resources.getString("mapWidth")));
		toolBar.add(width);
		toolBar.add(new JLabel(Resources.getString("mapLength")));
		toolBar.add(length);
		mapPanel = new JDesktopPane();
		mapPanel.setLayout(new GridBagLayout());
		mapPanel.add(map, new GridBagConstraints());
		mapPanel.setBackground(Color.GRAY);
		fond.setOpaque(false);
		fond.setLayout(new PercentageLayout());
		fond.add(menubar, new PercentageLayout.Constraint(0, 0, 100, 5));
		fond.add(mapPanel, new PercentageLayout.Constraint(0, 10, 100, 90));
		fond.add(toolBar, new PercentageLayout.Constraint(0, 5, 100, 5));
		contentPane.add(fond);
		fond.setSize(1024, 768);
		contentPane.add(fond);
		validate();
		setSize(1024, 768);
		repaint();
		hookupEvents();
		river.doClick();

	}

	/**
	 * change map width
	 * @param ev textfield event
	 */
	private void setMapWidth(EventObject ev){
		try {
            int mapWidth=Integer.parseInt(((JTextField)ev.getSource()).getText());
            MapModel.getInstance().setMapWidth(mapWidth); 
            Dimension d=new Dimension(MapModel.getInstance().getMapWidth(), MapModel.getInstance().getMapLength());
            map.setSize(d);
            map.setPreferredSize(d);
            map.setMinimumSize(d);
            validate();
            repaint();
        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
	}
	/**
	 * change map length
	 * @param ev textfield event
	 */
	private void setMapLength(EventObject ev){
		try {
            int mapLength=Integer.parseInt(((JTextField)ev.getSource()).getText());
            MapModel.getInstance().setMapLength(mapLength);
            Dimension d=new Dimension(MapModel.getInstance().getMapWidth(), MapModel.getInstance().getMapLength());
            map.setSize(d);
            map.setPreferredSize(d);
            map.setMinimumSize(d);
            validate();
            repaint();
        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
	}
	/**
	 * register events
	 *
	 */
	private void hookupEvents() {
       
        width.addKeyListener(new KeyAdapter() {
            public void keyReleased(KeyEvent ev) {
                setMapWidth(ev);
            }
        });
        width.addFocusListener(new FocusAdapter() {
        	public void focusLost(FocusEvent ev) {
                setMapWidth(ev);
            }
        });

        
        length.addKeyListener(new KeyAdapter() {
            public void keyReleased(KeyEvent ev) {
            	setMapLength(ev);
            }
        });
        length.addFocusListener(new FocusAdapter() {
        	public void focusLost(FocusEvent ev) {
                setMapLength(ev);
            }
        });     
        
		map.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				try {

					Point m = e.getPoint();
					Point p =
						new Point(
							(int) ((m.getX() - 0) / 1),
							(int) ((m.getY() - 0) / 1));


					//if (e.getButton() == MouseEvent.BUTTON1) {
					if (e.getModifiers() == MouseEvent.BUTTON1_MASK) {
						MapModel.getInstance().addPoint(p);
					} else {
						MapModel.getInstance().close();
					}

					map.repaint();

				} catch (NullPointerException npe) {
					npe.printStackTrace();
				}
			} //mouse clicked
		});



		//new

		newmap.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().reset();
				map.repaint();
			}
		});
		//open image

		openimage.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				
				FilePopup loadBox = new FilePopup("maps", "OpenMap", "MapName",
						false);
				mapPanel.add(loadBox, 0);
				validate();
				String mapFile = loadBox.getFile();
				
				if (mapFile != null) {
					String mapName = (new File(mapFile)).getName();
					map.setMapImage(mapName);
					MapModel.getInstance().setImageName(mapName);
				}


			}
		});
		
		//open map

		openmap.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				
				FilePopup loadBox = new FilePopup("mapmodels", "OpenMap", "MapName",
						false);
				mapPanel.add(loadBox, 0);
				String mapFile = loadBox.getFile();
				if (mapFile != null) {
					MapModel.getInstance().load(mapFile);
					map.setMapImage(MapModel.getInstance().getImageName());
					river.doClick();
					width.setText(""+MapModel.getInstance().getMapWidth());
					length.setText(""+MapModel.getInstance().getMapLength());
					Dimension d=new Dimension(MapModel.getInstance().getMapWidth(), MapModel.getInstance().getMapLength());
		            map.setSize(d);
		            map.setPreferredSize(d);
		            map.setMinimumSize(d);
		            validate();
		            repaint();
				}


			}
		});
		
		//undo

		undoLast.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {

				MapModel.getInstance().undo();
				map.repaint();

			}
		});

		//save

		savemap.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {

				if (MapModel.getInstance().getPath() == null) {

					
					FilePopup saveBox = new FilePopup("mapmodels", "SaveMap", "MapName",
							true);
					mapPanel.add(saveBox, 0);
					String mapFile = saveBox.getFile();
					if (mapFile != null) {
						MapModel.getInstance().save(mapFile);
					}					
					
				} else {
					MapModel.getInstance().save();
				}
			}
		});

		//save as

		saveasmap.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {

				FilePopup saveBox = new FilePopup("mapmodels", "SaveMap", "MapName",
						true);
				mapPanel.add(saveBox, 0);
				String mapFile = saveBox.getFile();
				if (mapFile != null) {
					MapModel.getInstance().save(mapFile);
				}

			}
		});

		//exit

		exit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {

				System.exit(0);

			}
		});

		river.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('r');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});

		wood.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('w');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});

		marais.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('m');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});
		house.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('h');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});
		fog.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('g');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});
		JMenuItem item;
		for (int i=0;i<vsets.length;i++){
			item=vsets[i];
			item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				JMenuItem item=(JMenuItem)ev.getSource();
				MapModel.getInstance().setCurrenttype(item.getText().charAt(0));
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});
		}

		
		
		aSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('a');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});

		bSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('b');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});

		cSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('c');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});
		dSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('d');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});
		eSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('e');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});		
		fSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MapModel.getInstance().setCurrenttype('f');
				currentTypeLabel.setText(Resources.getString("currenttype")+((JMenuItem)ev.getSource()).getText());
			}
		});			

	} //hookupevents

}