/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fb.gui.widgets.FBComboBox;
import fb.gui.widgets.SelectableListener;
import fb.main.FB;
import fb.network.Client;
import fb.network.Server;
import fb.util.ComboArrayListModel;
import fb.util.FBColors;
import fb.util.FileArrayListModel;
import fb.util.Resources;
/**
 * a panel that contains a combo box and a campaign minimap
 * in order to choose the map for campaign game 
 * @author farfpeuf
 *
 */
public class CampaignChooser extends JPanel {
	private static File campaignsDirectory = new File("campaigns");

	//the combo box to choose map
	private FBComboBox campaignCombo;
	//the label next to the combo
	private JLabel campaignNameLabel;
	//the 2 singleton instances (editable and non editable)
	private static CampaignChooser editableInstance, nonEditableInstance;
	//the mini campaing map panel to display chosen map
	private MiniHexagonalMapPanel campaignMap;

	//selected campaign name
	private String campaignName;
	/**
	 * get singleton instance
	 * @return the singleton
	 */
	public static CampaignChooser getInstance() {
		return getInstance(Server.hasInstance()|| !FB.isNet());
	}
	/**
	 * get singleton instance
	 * @param editableMode editable or non editable instance ?
	 * @return the singleton
	 */
	public static CampaignChooser getInstance(boolean editableMode) {
		if (editableMode) {
			if (null == editableInstance) {
				editableInstance = new CampaignChooser(editableMode);
			}
			return editableInstance;
		} else {
			if (null == nonEditableInstance) {
				nonEditableInstance = new CampaignChooser(editableMode);
			}
			return nonEditableInstance;
		}
	}
	/**
	 * Panel constructor
	 * @param editableMode editable or non editable ?
	 */
	private CampaignChooser(boolean editableMode) {
		setLayout(new GridBagLayout());
		setBackground(FBColors.background);

		campaignMap=new MiniHexagonalMapPanel();
		
		GridBagConstraints gbc=new GridBagConstraints();
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.insets=new Insets(5,5,5,5);
		gbc.weightx=1;
		gbc.weighty=1;
		add(campaignMap, gbc);
		gbc.gridy++;
		gbc.weighty=0;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		if (editableMode) {
			ComboBoxModel model=new ComboArrayListModel(FileArrayListModel.getInstance("campaigns"));
			campaignCombo = new FBComboBox(model);
			if (model.getSize()>0){
				campaignCombo.setSelectedIndex(0);
			}
			changeCampaign();
			campaignCombo.addItemListener(new SelectableListener() {
				public void threadedItemStateChanged() {
					changeCampaign();
				}
			});
			add(campaignCombo, gbc);
		} else {
			campaignNameLabel = new JLabel();
			campaignNameLabel.setOpaque(false);
			campaignNameLabel.setForeground(FBColors.primary);
			campaignNameLabel.setHorizontalAlignment(JLabel.CENTER);
			add(campaignNameLabel, gbc);
		}

		setBorder(BorderFactory.createLineBorder(FBColors.darkBorder));
		validate();
		repaint();
        setToolTipText(Resources.getString("field.tt"));
	}
	
	/**
	 * handle combo change
	 *
	 */
	private void changeCampaign(){
		setCampaign((String) campaignCombo.getSelectedItem());
		if (FB.isNet()) {
			Client.getInstance().send("SETCAMPAIGN"+ campaignName);
		}			
	}
	/**
	 * get selected map
	 * @return map name
	 */
	public String getSelectedMap() {
			return campaignName;
	}
	/**
	 * change selected map
	 * @param newName the new map
	 */
	public void setCampaign(String newName) {
		if (!newName.equals(campaignName)) {
			campaignName=newName;
			campaignMap.init(campaignName,false);
		}
	}


}