/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;

import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.EventObject;
import java.util.List;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextField;


import fb.gui.frames.popups.FilePopup;
import fb.gui.widgets.ButtonListener;
import fb.main.FB;
import fb.util.FBColors;
import fb.util.FileTools;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * frame used to create and edit campaign maps
 * 
 * @author farfpeuf
 * 
 */
public class CampaignMapEditor extends JPanel {

	// current tile drawn when dragging mouse on the map
	private char currentTile;

	// the amp panel
	private HexagonalMapPanel map;

	// current type label
	private JLabel currentTypeLabel;

	// menus
	private JMenu mainmenu, tilesetMenu, sideMenu;

	// sub menus
	private JMenuItem exit, newmap, openmap, savemap, saveasmap, aSide, bSide,
			cSide, dSide, eSide, fSide;

	// fields to modify map width and length
	private JTextField width, length;

	// available tilesets sub menus
	private TileMenuItem[] tilesets;

	// singleton instance
	private static CampaignMapEditor instance;

	/**
	 * class for available tilesets menu items
	 * 
	 * @author farfpeuf
	 * 
	 */
	private class TileMenuItem extends JMenuItem {

		private char tile;

		public TileMenuItem(char tile) {
			super(Resources.getTileImage(tile));
			this.tile = tile;
		}

		public char getTile() {
			return tile;
		}

	}

	/**
	 * start map
	 * (handle mouse on frame edges)
	 */
	public void start() {
		map.start();
	}

	/**
	 * constructor build frame and hookup events
	 */
	private CampaignMapEditor() {
		setBackground(FBColors.background);
		buildFrame();
		hookupEvents();
	}

	/**
	 * build and assemble frame components
	 *
	 */
	private void buildFrame() {
		map = new HexagonalMapPanel();
		map.setModel(HexagonalMapModel.getInstance());
		map.setRadius(30);
		currentTypeLabel = new JLabel();
		JMenuBar menubar;
		menubar = new JMenuBar();
		menubar.setForeground(FBColors.standardText);
		menubar.setBackground(FBColors.background);
		width = new JTextField(""
				+ HexagonalMapModel.getInstance().getMapWidth());
		length = new JTextField(""
				+ HexagonalMapModel.getInstance().getMapLength());
		// build menus

		mainmenu = new JMenu(Resources.getString("Map"));
		tilesetMenu = new JMenu(Resources.getString("Tilesets"));
		sideMenu = new JMenu(Resources.getString("Side"));

		mainmenu.setForeground(FBColors.standardText);
		mainmenu.setBackground(FBColors.background);
		tilesetMenu.setForeground(FBColors.standardText);
		tilesetMenu.setBackground(FBColors.background);
		sideMenu.setForeground(FBColors.standardText);
		sideMenu.setBackground(FBColors.background);

		newmap = new JMenuItem(Resources.getString("Newmap"));
		openmap = new JMenuItem(Resources.getString("Openmap"));
		savemap = new JMenuItem(Resources.getString("Save"));
		saveasmap = new JMenuItem(Resources.getString("Saveas"));
		exit = new JMenuItem(Resources.getString("Exit"));
		mainmenu.add(newmap);
		mainmenu.add(openmap);
		mainmenu.add(savemap);
		mainmenu.add(saveasmap);
		mainmenu.add(exit);

		List tilesetsList = FileTools.list("tile");

		if (tilesetsList != null && !tilesetsList.isEmpty()) {
			char tileChar;
			TileMenuItem item;

			tilesets = new TileMenuItem[tilesetsList.size()];
			int index = 0;
			String tileFileName;
			for (int i = 0; i < tilesets.length; i++) {

				tileFileName = (String) tilesetsList.get(i);
				tileChar = tileFileName.charAt(0);
				if (tileChar != 'T') {
					item = new TileMenuItem(tileChar);
					tilesets[i] = item;
					tilesetMenu.add(item);
					index++;
				}

			}
		}

		aSide = new JMenuItem(Resources.getString("Side") + " A");
		bSide = new JMenuItem(Resources.getString("Side") + " B");
		cSide = new JMenuItem(Resources.getString("Side") + " C");
		dSide = new JMenuItem(Resources.getString("Side") + " D");
		eSide = new JMenuItem(Resources.getString("Side") + " E");
		fSide = new JMenuItem(Resources.getString("Side") + " F");
		sideMenu.add(aSide);
		sideMenu.add(bSide);
		sideMenu.add(cSide);
		sideMenu.add(dSide);
		sideMenu.add(eSide);
		sideMenu.add(fSide);
		menubar.add(mainmenu);
		menubar.add(tilesetMenu);
		menubar.add(sideMenu);
		JPanel toolBar = new JPanel();
		toolBar.setLayout(new FlowLayout());
		toolBar.add(currentTypeLabel);
		toolBar.add(new JLabel(Resources.getString("mapWidth")));
		toolBar.add(width);
		toolBar.add(new JLabel(Resources.getString("mapLength")));
		toolBar.add(length);

		setLayout(new PercentageLayout());
		add(menubar, new PercentageLayout.Constraint(0, 0, 100, 5));
		add(map, new PercentageLayout.Constraint(0, 20, 100, 90));
		add(toolBar, new PercentageLayout.Constraint(0, 5, 100, 15));
		setCurrentTile(HexagonalMapModel.DEFAULT_TILE);
		validate();
		repaint();

	}

	/**
	 * get singleton instance
	 * @return the singleton
	 */
	public static synchronized CampaignMapEditor getInstance() {
		if (instance == null) {
			instance = new CampaignMapEditor();
		}
		return instance;
	}

	/**
	 * change map width by modifying textfield values
	 * @param ev textefield event
	 */
	private void setMapWidth(EventObject ev) {
		try {
			int mapWidth = Integer.parseInt(((JTextField) ev.getSource())
					.getText());
			HexagonalMapModel.getInstance().setMapWidth(mapWidth);
			validate();
			repaint();
		} catch (NumberFormatException e) {
			// ignore
		}
	}
	/**
	 * change map length by modifying textfield values
	 * @param ev textefield event
	 */
	private void setMapLength(EventObject ev) {
		try {
			int mapLength = Integer.parseInt(((JTextField) ev.getSource())
					.getText());
			HexagonalMapModel.getInstance().setMapLength(mapLength);
			validate();
			repaint();
		} catch (NumberFormatException e) {
			// ignore
		}
	}

	/**
	 * register event listeners
	 *
	 */
	private void hookupEvents() {

		width.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ev) {
				setMapWidth(ev);
			}
		});
		width.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent ev) {
				setMapWidth(ev);
			}
		});

		length.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ev) {
				setMapLength(ev);
			}
		});
		length.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent ev) {
				setMapLength(ev);
			}
		});

		map.addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseDragged(MouseEvent e) {
				try {
					changeTile(e.getPoint());
				} catch (NullPointerException npe) {
					npe.printStackTrace();
				}
			} // mouse clicked
		});

		map.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				try {
					changeTile(e.getPoint());
				} catch (NullPointerException npe) {
					npe.printStackTrace();
				}
			} // mouse clicked

		});

		// new

		newmap.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				HexagonalMapModel.getInstance().reset();
				map.refreshAll();
			}
		});
		// open image

		// open map

		openmap.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {

				FilePopup loadBox = new FilePopup("campaigns", "OpenMap",
						"MapName", false);
				FB.getInstance().getContentPane().add(loadBox, 0);
				String mapFile = loadBox.getFile();
				if (mapFile != null) {
					HexagonalMapModel.getInstance().reset();
					HexagonalMapModel.getInstance().loadFromFile(mapFile);
					width.setText(""
							+ HexagonalMapModel.getInstance().getMapWidth());
					length.setText(""
							+ HexagonalMapModel.getInstance().getMapLength());
					validate();
					repaint();
				}

			}
		});

		// save

		savemap.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {

				if (HexagonalMapModel.getInstance().getPath() == null) {

					FilePopup saveBox = new FilePopup("campaigns", "SaveMap",
							"MapName", true);
					FB.getInstance().getContentPane().add(saveBox, 0);
					String mapFile = saveBox.getFile();
					if (mapFile != null) {
						HexagonalMapModel.getInstance().save(mapFile);
					}

				} else {
					HexagonalMapModel.getInstance().save();
				}
			}
		});

		// save as

		saveasmap.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {

				FilePopup saveBox = new FilePopup("campaigns", "SaveMap",
						"MapName", true);
				FB.getInstance().getContentPane().add(saveBox, 0);
				String mapFile = saveBox.getFile();
				if (mapFile != null) {
					HexagonalMapModel.getInstance().save(mapFile);
				}

			}
		});

		// exit

		exit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				map.stop();

				HexagonalMapModel.getInstance().reload();

				FB.getInstance().displayCampaignPreparationFrame();
			}
		});

		TileMenuItem item;
		for (int i = 0; i < tilesets.length; i++) {
			item = tilesets[i];
			if (item != null) {
				item.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ev) {
						TileMenuItem item = (TileMenuItem) ev.getSource();
						setCurrentTile(item.getTile());
						currentTypeLabel.setText("");
					}
				});
			}
		}

		aSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				setCurrentTile('A');
				currentTypeLabel
						.setText(((JMenuItem) ev.getSource()).getText());
			}
		});

		bSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				setCurrentTile('B');
				currentTypeLabel
						.setText(((JMenuItem) ev.getSource()).getText());
			}
		});

		cSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				setCurrentTile('C');
				currentTypeLabel
						.setText(((JMenuItem) ev.getSource()).getText());
			}
		});
		dSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				setCurrentTile('D');
				currentTypeLabel
						.setText(((JMenuItem) ev.getSource()).getText());
			}
		});
		eSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				setCurrentTile('E');
				currentTypeLabel
						.setText(((JMenuItem) ev.getSource()).getText());
			}
		});
		fSide.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				setCurrentTile('F');
				currentTypeLabel
						.setText(((JMenuItem) ev.getSource()).getText());
			}
		});

	} // hookupevents

	
	/**
	 * change tile under mouse cursor
	 * @param mouse mouse coordinate
	 */
	
	public void changeTile(Point mouse) {
		Point m = map.getHexaCoord(mouse);

		if (currentTile >= 'A' && currentTile <= 'F') {
			Point oldCapital = (Point) HexagonalMapModel.getInstance()
					.getCapital(currentTile - 'A');
			if (oldCapital != null) {
				HexagonalMapModel.getInstance().setTile(oldCapital.x,
						oldCapital.y, HexagonalMapModel.DEFAULT_TILE);
				map.refresh(map.getHexaRectangle(oldCapital));
			}
		}

		HexagonalMapModel.getInstance().setTile(m.x, m.y, currentTile);
		map.refresh(map.getHexaRectangle(m));

	}

	/**
	 * change tile type drawn when dragging mouse on the map
	 * @param currentTile the new tile type
	 */
	public void setCurrentTile(char currentTile) {
		this.currentTile = currentTile;
		Icon icon;
		if (currentTile >= 'A' && currentTile <= 'F') {
			icon = null;
		} else {
			icon = Resources.getTileImage(currentTile);
		}
		currentTypeLabel.setIcon(icon);
	}

}