/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;

import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import fb.army.hierarchy.Force;
import fb.gui.panels.StatePanel;
import fb.gui.panels.TreeTablePanel;
import fb.gui.widgets.armytreetable.ArmyTreeModel;
import fb.gui.widgets.armytreetable.TreeTable;
import fb.gui.widgets.armytreetable.TreeTableModelAdapter;
import fb.util.Resources;

/**
 * panel containing some information sub panels for campaign game
 * @author Farfpeuf
 */
public class CampaignStatePanel extends StatePanel {

	//singleton instance
	private static CampaignStatePanel instance;

	//tree table panels for each forces involved
	private TreeTablePanel[] forcesPanel;

	/**
	 * get singleton instance
	 * @return the singleton
	 */
	public static CampaignStatePanel getInstance() {
		if (null == instance) {
			instance = new CampaignStatePanel();
		}
		return instance;
	}

	/**
	 * update one force trre table panel
	 * @param force the force
	 */
	public void updateForceTab(Force force) {
		if (forcesPanel != null  && force!=null) {
			TreeTablePanel panel = forcesPanel[force.getId()];
			TreeTableModelAdapter ad = panel.getArmyTableModel();
			ArmyTreeModel model = (ArmyTreeModel) ad.getTreeTableModel();
			model.nodeChanged(force);
			model.nodeStructureChanged(force);
		}
	}

	/**
	 * build all panels tabs
	 */
	public void buildTabs() {
		super.buildTabs();
		Force force;
		forcesPanel = new TreeTablePanel[Campaign.getInstance().getForces().length];
		for (int i = 0; i < forcesPanel.length; i++) {
			force = Campaign.getInstance().getForces()[i];
			forcesPanel[i] = new TreeTablePanel(force, true);
			forcesPanel[i].getTreeTable().setColumnWidths(new int[]{200,80,150});
			forcesPanel[i].setToolTipText(Resources.getString("playetree.tt"));
			forcesPanel[i].getTreeTable().setToolTipText(
					Resources.getString("armytree.tt"));
			tabbedPane.addTab(force.getPlayerName(), forcesPanel[i]);

			final TreeTable table = forcesPanel[i].getTreeTable();

			table.getSelectionModel().addListSelectionListener(
					new ListSelectionListener() {
						public void valueChanged(ListSelectionEvent e) {
							ListSelectionModel model = (ListSelectionModel) e
									.getSource();

							TreeTableModelAdapter ad = (TreeTableModelAdapter) table.getModel();
							Object leadNode = ad.nodeForRow(model
									.getLeadSelectionIndex());

							CampaignGameFrame.getInstance().centerOn(leadNode);

						}
					});

		}

	}


}
