/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;


import fb.army.hierarchy.Army;
import fb.gui.frames.popups.ComboPopup;
import fb.gui.frames.popups.FilePopup;
import fb.gui.panels.ArmyEditorPanel;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBToolBarButton;
import fb.main.FB;
import fb.util.FBColors;
import fb.util.FileArrayListModel;
import fb.util.FileTools;
import fb.util.PercentageLayout;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * frame containing the army editor
 * 
 * @author farfpeuf
 * 
 */
public class ArmyEditorFrame extends JPanel {

	/**
	 * buttons
	 */
	private JButton newArmy, openArmy, saveArmy, saveAsArmy, exit;

	/**
	 * constructor
	 * 
	 */
	public ArmyEditorFrame() {
		buildFrame();
		ArmyEditorPanel.getInstance().setInitialBudget(1500);
	}

	/**
	 * build and assemble elements
	 * 
	 */
	private void buildFrame() {

		if (FB.getInstance().isMini()) {
			newArmy = new FBButton("new");
			openArmy = new FBButton("open");
			saveArmy = new FBButton("save");
			saveAsArmy = new FBButton("saveas");
			exit = new FBButton("exit");
		} else {
			newArmy = new FBToolBarButton("new");
			openArmy = new FBToolBarButton("open");
			saveArmy = new FBToolBarButton("save");
			saveAsArmy = new FBToolBarButton("saveas");
			exit = new FBToolBarButton("exit");
		}

		newArmy.setToolTipText(Resources.getString("Newarmy"));
		openArmy.setToolTipText(Resources.getString("Openarmy"));
		saveArmy.setToolTipText(Resources.getString("Save"));
		saveAsArmy.setToolTipText(Resources.getString("Saveas"));
		exit.setToolTipText(Resources.getString("Exittomainmenu"));

		setLayout(new PercentageLayout());
		setOpaque(false);
		JPanel buttonsPanel = new JPanel(new GridBagLayout());
		buttonsPanel.setBorder(BorderFactory
				.createLineBorder(FBColors.darkBorder));
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.weightx = 1;
		buttonsPanel.add(newArmy, gbc);
		buttonsPanel.add(openArmy, gbc);
		buttonsPanel.add(saveArmy, gbc);
		buttonsPanel.add(saveAsArmy, gbc);
		buttonsPanel.add(exit, gbc);
		add(buttonsPanel, new PercentageLayout.Constraint(10, 2, 15, 4));
		add(ArmyEditorPanel.getInstance(), new PercentageLayout.Constraint(5, 5,
				90, 90));

		setSize((int) (800 * FB.getXres()), (int) (600 * FB.getYres()));

		hookupEvents();

	}

	/**
	 * create an army
	 * 
	 */
	public void createAction() {
		String[] racelist = new String[] { Resources.getString("aebarbarian"),
				Resources.getString("aegoblin"),
				Resources.getString("aedwarf"),
				Resources.getString("aewoodelf"),
				Resources.getString("aehuman"), Resources.getString("aeorc"),
				Resources.getString("aehigh-elf") };

		ComboPopup createBox = new ComboPopup(racelist, "createarmy",
				"Choosearmyrace", false);
		add(createBox, new PercentageLayout.Constraint(20, 20, 30, 12), 0);
		String chosenRace = createBox.getString();

		if (null != chosenRace) {

			String race = Resources.getString(StringTools.replaceAll(
					chosenRace, " ", ""));
			Army newArmy = new Army(race, "armies" + File.separator
					+ StringTools.replaceAll(chosenRace, " ", ""));
			ArmyEditorPanel.getInstance().setArmy(newArmy);
		}

	}

	/**
	 * edit an army
	 * 
	 */
	public void editAction() {

		FilePopup editbox = new FilePopup("armies", "editarmy", "Choosearmy",
				false);
		add(editbox, new PercentageLayout.Constraint(20, 20, 30, 12), 0);
		String armyFile = editbox.getFile();
		if (null != armyFile) {
			ArmyEditorPanel.getInstance().openArmy(armyFile);

		}

	}

	/**
	 * save an army as
	 * 
	 */
	public void saveAsAction() {

		FilePopup saveAsBox = new FilePopup("armies", "Saveas", "Saveas", true);
		if (FB.getInstance().isMini()) {
			add(saveAsBox, new PercentageLayout.Constraint(20, 20, 30, 30), 0);
		} else {
			add(saveAsBox, new PercentageLayout.Constraint(20, 20, 30, 12), 0);
		}
		String armyFile = saveAsBox.getFile();
		if (null != armyFile) {
			FileTools.writeFile(armyFile, ArmyEditorPanel.getInstance()
					.getArmy().saveArmyString());
			FileArrayListModel.refresh("armies");
		}

	}

	/**
	 * register events
	 * 
	 */
	private void hookupEvents() {

		// --------------------------------------------------------------------------------
		// menus
		// ---------------------------------------------------------------------------------
		// new
		newArmy.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				createAction();
			}
		});
		// open
		openArmy.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				editAction();
			}
		});
		// save
		saveArmy.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				Army army = ArmyEditorPanel.getInstance().getArmy();
				String message = army.check(ArmyEditorPanel.getInstance()
						.getInitialBudget());
				if (message.equals("")) {
					FileTools.writeFile(army.getFile(), army.saveArmyString());
					FileArrayListModel.refresh("armies");
				} else {
					FB.getInstance().popUpMessage(
							Resources.getString("cannotSaveArmy"), message);
				}
			}
		});
		// save as
		saveAsArmy.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				Army army = ArmyEditorPanel.getInstance().getArmy();
				String message = army.check(ArmyEditorPanel.getInstance()
						.getInitialBudget());
				if (message.equals("")) {
					saveAsAction();
				} else {
					FB.getInstance().popUpMessage(
							Resources.getString("cannotSaveArmy"), message);
				}
			}
		});
		// exit
		exit.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().displayMainMenuFrame();
			}
		});
	} // hookupevents

}