/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Robot;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Calendar;

import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.KeyStroke;

import fb.army.entities.Druid;
import fb.army.entities.Entity;
import fb.army.entities.Individual;
import fb.army.entities.Priest;
import fb.army.entities.Unit;
import fb.army.entities.Wizard;
import fb.army.hierarchy.Army;
import fb.army.hierarchy.Command;
import fb.battlefield.BattleFieldPanel;
import fb.battlefield.MiniBattleFieldMapPanel;
import fb.gui.panels.BattleStatePanel;
import fb.gui.panels.DruidSpellPanel;
import fb.gui.panels.EntityBattlePanel;
import fb.gui.panels.ExpandPanel;
import fb.gui.panels.IndividualManoeuvrePanel;
import fb.gui.panels.OrderPanel;
import fb.gui.panels.PhasePanel;
import fb.gui.panels.PriestSpellPanel;
import fb.gui.panels.ReducePanel;
import fb.gui.panels.SpellPanel;
import fb.gui.panels.UnitManoeuvrePanel;
import fb.gui.panels.WizardSpellPanel;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.main.FB;
import fb.main.Battle;
import fb.phases.BinaryPhase;
import fb.phases.DeploymentPhase;
import fb.phases.Phase;
import fb.phases.PlayerMagicPhase;
import fb.phases.PlayerOrderPhase;
import fb.util.FBColors;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * frame for battles
 * 
 * @author farfpeuf
 * 
 */
public class BattleFrame extends JPanel {

	/**
	 * maximum view scale
	 */
	private static int MAX_SCALE = FB.getInstance().isDebug() ? 8 : 7; // TODO
																		// fix 8
																		// scale
																		// bug

	/**
	 * buttons
	 */
	private FBButton nextPhaseButton, zoomin, zoomout, quit, magic, newOrders,
			rout, saveGame;

	/**
	 * singleton instance
	 */
	private static BattleFrame instance;

	/**
	 * battlefield
	 */
	private BattleFieldPanel battlefieldPanel;

	/**
	 * mini battlefield map
	 */
	private MiniBattleFieldMapPanel minimap;

	/**
	 * entity statistics
	 */
	private EntityBattlePanel entityStatPanel = new EntityBattlePanel();

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static synchronized BattleFrame getInstance() {
		if (null == instance) {
			instance = new BattleFrame();
		}
		return instance;
	}

	/**
	 * sub panels
	 */
	private JPanel actionpanel, textButtonsPanel;

	/**
	 * sub panels with background images
	 */
	private JLabel northpanel, southpanel;

	/**
	 * action panels
	 */

	private WizardSpellPanel wizardPanel;

	private DruidSpellPanel drdPanel;

	private PriestSpellPanel priestPanel;

	private OrderPanel orderpanel;

	private PhasePanel phasepanel;

	private UnitManoeuvrePanel unitMovementPanel;

	private ExpandPanel expandPanel;

	private ReducePanel reducePanel;

	private IndividualManoeuvrePanel indvdMovementPanel;

	private SpellPanel spellPanel;

	/**
	 * constructor build and assemble elements
	 * 
	 */
	private BattleFrame() {

		setOpaque(false);

		setBackground(FBColors.background);

		southpanel = new JLabel(Resources.getLookNFeelImage("frise"));
		textButtonsPanel = new JPanel();
		JPanel zoombuttons = new JPanel();

		textButtonsPanel.setLayout(new GridLayout(5, 1));

		zoombuttons.setLayout(new GridLayout(2, 1));
		southpanel.setLayout(new PercentageLayout());
		battlefieldPanel = new BattleFieldPanel();
		minimap = new MiniBattleFieldMapPanel();

		actionpanel = new JPanel();
		actionpanel.setLayout(null);
		actionpanel.setOpaque(false);
		nextPhaseButton = new FBButton(Resources.getString("nextphasebutton"));
		zoomin = new FBButton(null, "zoomin", true);
		zoomout = new FBButton(null, "zoomout", true);
		quit = new FBButton(Resources.getString("Exittomainmenu"));
		saveGame = new FBButton(Resources.getString("saveGame"));
		magic = new FBButton(Resources.getString("magicbutton"));
		newOrders = new FBButton(Resources.getString("orderbutton"));
		rout = new FBButton(Resources.getString("Escape"));

		rout.setToolTipText(Resources.getString("escape.tt"));
		nextPhaseButton.setToolTipText(Resources.getString("endphase.tt"));
		zoomin.setToolTipText(Resources.getString("zoomin.tt"));
		zoomout.setToolTipText(Resources.getString("zoomout.tt"));
		quit.setToolTipText(Resources.getString("backtomenu.tt"));
		saveGame.setToolTipText(Resources.getString("save.tt"));
		magic.setToolTipText(Resources.getString("magicbutton.tt"));
		newOrders.setToolTipText(Resources.getString("neworderbutton.tt"));

		nextPhaseButton.setEnabled(false);
		magic.setEnabled(false);
		newOrders.setEnabled(false);
		saveGame.setEnabled(false);
		rout.setEnabled(false);
		zoombuttons.add(zoomin);
		zoombuttons.add(zoomout);

		// build northpanel
		northpanel = new JLabel(Resources.getLookNFeelImage("frise"));
		northpanel.setLayout(new PercentageLayout());

		northpanel.add(BattleStatePanel.getInstance(),
				new PercentageLayout.Constraint(0, 0, 50, 100));
		northpanel.add(entityStatPanel, new PercentageLayout.Constraint(50, 0, 50,
				100));

		// build minimappanel
		GridBagConstraints gbc = new GridBagConstraints();
		JPanel miniMapPanel = new JPanel();
		miniMapPanel.setLayout(new GridBagLayout());
		gbc = new GridBagConstraints();
		miniMapPanel.add(minimap, gbc);

		// build southpanel
		southpanel.add(zoombuttons, new PercentageLayout.Constraint(0, 0, 6, 100));

		southpanel
				.add(miniMapPanel, new PercentageLayout.Constraint(7, 1, 18, 98));
		southpanel.add(textButtonsPanel, new PercentageLayout.Constraint(26, 0,
				12, 100));

		southpanel.add(actionpanel,
				new PercentageLayout.Constraint(38, 0, 62, 100));

		if (FB.getInstance().isMini()) {
			setLayout(new PercentageLayout());
			add(battlefieldPanel, new PercentageLayout.Constraint(0, 0, 100, 80));
			add(southpanel, new PercentageLayout.Constraint(0, 80, 100, 20));
		} else {
			setLayout(new GridBagLayout());
			// build frame
			gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.weightx = 1;
			gbc.fill = GridBagConstraints.BOTH;
			add(northpanel, gbc);
			gbc.weighty = 1;
			gbc.gridy++;
			add(battlefieldPanel, gbc);
			gbc.weighty = 0;
			gbc.gridy++;
			add(southpanel, gbc);
		}
		validate();
		setBounds((int) (FB.getXres() * 5), (int) (FB.getYres() * 5), (int) (FB
				.getXres() * 790), (int) (FB.getYres() * 590));
		hookupEvents();

	}

	/**
	 * initialize frame
	 * 
	 * @param mapName
	 *            map name
	 * @param mapwidth
	 *            map width
	 * @param mapLength
	 *            map length
	 * @param flipLR
	 *            flip map from left to right?
	 * @param flipUD
	 *            flip map from up to down?
	 */
	public void init(String mapName, int mapwidth, int mapLength,
			boolean flipLR, boolean flipUD) {
		battlefieldPanel.init(mapName, mapwidth, mapLength, flipLR, flipUD);
		minimap.init(mapName, mapwidth, mapLength);
	}

	/**
	 * change action panel
	 * 
	 * @param c
	 *            the new action panel
	 */
	public void setActionPanel(Component c) {
		actionpanel.removeAll();
		actionpanel.add(c);
		actionpanel.validate();
		actionpanel.repaint();
	}

	/**
	 * handle buttons enbling
	 * 
	 * @param enable
	 *            enable or disable button ?
	 */
	public void enableButtons(boolean enable) {
		magic.setEnabled(enable);
		newOrders.setEnabled(enable);
		nextPhaseButton.setEnabled(enable);
		saveGame.setEnabled(enable);
		rout.setEnabled(enable);
	}

	/**
	 * clear action panel
	 * 
	 */
	public void clearActionPanel() {
		actionpanel.removeAll();
		actionpanel.repaint();
	}

	/**
	 * displa an entity characteristics
	 * 
	 * @param e
	 *            the entity
	 */
	public void displayEntityCharacteristics(Entity e) {
		entityStatPanel.display(e);
	}

	/**
	 * register events
	 * 
	 */
	private void hookupEvents() {
		FB.getInstance().requestFocus();

		battlefieldPanel.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				try {
					Point m = e.getPoint();
					final Point p = new Point(
							(int) ((m.getX() - battlefieldPanel.getDx()) / battlefieldPanel
									.getScale()),
							(int) ((m.getY() - battlefieldPanel.getDy()) / battlefieldPanel
									.getScale()));
					Entity entity;
					// if (e.getButton() == MouseEvent.BUTTON1) {
					if (e.getModifiers() == MouseEvent.BUTTON1_MASK) {
						if (Battle.getLoop().currentSynchronizedPhase != null
								&& Battle.getLoop().currentSynchronizedPhase instanceof BinaryPhase) {

							Thread t = new Thread("player click") {
								public void run() {
									((BinaryPhase) Battle.getLoop().currentSynchronizedPhase)
											.playerClicked(p);
								}
							};
							t.start();

						}
					}
					if (!(Battle.getLoop().currentSynchronizedPhase instanceof DeploymentPhase)) {
						for (int i = 0; i < Battle.getInstance()
								.getActiveEntities().size(); i++) {
							entity = (Entity) (Battle.getInstance()
									.getActiveEntities().get(i));
							if (entity.getArea().contains(p)) {
								displayEntityCharacteristics(entity);
							}
						}
					}
				} catch (NullPointerException npe) {
					npe.printStackTrace();
				}
			} // mouse clicked
		});
		minimap.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				try {
					int minipointx, minipointy, maxipointx, maxipointy;
					minipointx = e.getX();
					minipointy = e.getY();
					maxipointx = (int) (minipointx * (((double) battlefieldPanel
							.getMapwidth()) / minimap.getWidth()));
					maxipointy = (int) (minipointy * (((double) battlefieldPanel
							.getMaplength()) / minimap.getMiniLength()));
					battlefieldPanel.centerOn(maxipointx, maxipointy);
				} catch (NullPointerException npe) {
					npe.printStackTrace();
				}
			} // mouse clicked
		});
		battlefieldPanel.addMouseWheelListener(new MouseWheelListener() {
			public void mouseWheelMoved(MouseWheelEvent e) {
				int roll = e.getWheelRotation();
				zoom(roll);
			} // mouse wheel moved
		});

		newOrders.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				magic.setEnabled(false);
				newOrders.setEnabled(false);
				saveGame.setEnabled(false);
				Battle.getLoop().interruptWith(new PlayerOrderPhase());
			}
		});
		magic.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				magic.setEnabled(false);
				newOrders.setEnabled(false);
				saveGame.setEnabled(false);
				Battle.getLoop().interruptWith(new PlayerMagicPhase());

			}
		});
		nextPhaseButton.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				try {
					if (Battle.getLoop().currentSynchronizedPhase != null
							&& Battle.getLoop().currentSynchronizedPhase instanceof BinaryPhase) {
						((BinaryPhase) Battle.getLoop().currentSynchronizedPhase)
								.playerClicked(null);
					}
				} catch (NullPointerException npe) {
					npe.printStackTrace();
				}
			}
		});
		rout.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				if (Battle.getLoop().currentSynchronizedPhase != null
						&& Battle.getLoop().currentSynchronizedPhase instanceof BinaryPhase) {

					BinaryPhase phase = ((BinaryPhase) Battle.getLoop().currentSynchronizedPhase);

					Army arm = phase.getCurrentArmy();
					if (arm != null) {
						arm.rout();
						phase.playerClicked(null);
					}
				}

			}
		});
		quit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				FB.getInstance().displayMainMenuFrame();
				Battle.getInstance().stop();
			}
		});
		saveGame.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				saveGame.setEnabled(false);
				if (Battle.getInstance().isCampaignGame()) {
					FB.getInstance().saveCampaign(true);
				} else {
					FB.getInstance().saveGame();
				}
				saveGame.setEnabled(true);
			}
		});
		zoomin.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				zoom(-1);
			}
		});
		zoomout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				zoom(1);
			}
		});
		InputMap inputMap = getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap actionMap = getActionMap();

		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0, false), "help");
		actionMap.put("help", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				BattleStatePanel.getInstance().showHelp();
			}
		});

		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0, false),
				"switchTransparentVSet");
		actionMap.put("switchTransparentVSet", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				battlefieldPanel.switchTransparentVSet();
			}
		});
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F3, 0, false),
				"switchDisplayPerformance");
		actionMap.put("switchDisplayPerformance", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				battlefieldPanel.switchDisplayPerformance();
			}
		});

		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false),
				"bossKey");
		actionMap.put("bossKey", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				FB.safeExit();
			}
		});

		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F4, 0, false),
				"switchPaintMap");
		actionMap.put("switchPaintMap", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				battlefieldPanel.switchPaintMap();
			}
		});
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0, false),
				"switchPaintLight");
		actionMap.put("switchPaintLight", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				battlefieldPanel.switchPaintLight();
			}
		});

		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0, false),
				"screenshot");
		actionMap.put("screenshot", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				try {
					BufferedImage bufferedImage = (new Robot())
							.createScreenCapture(battlefieldPanel.getBounds());
					File file = new File("screenshot"
							+ Calendar.getInstance().getTimeInMillis() + ".png");
					FileOutputStream out = new FileOutputStream(file);
					ImageIO.write(bufferedImage, "png", out);
					FB.getInstance().message(file.getPath());
				} catch (Exception ex) {
					FB.exception(ex);
				}
			}
		});

	} // hookupevents

	/**
	 * zoom in and out the battlefield
	 * 
	 * @param value
	 *            added or removed value
	 */
	public void zoom(int value) {
		int cx = ((battlefieldPanel.getWidth() / 2) - battlefieldPanel.getDx())
				/ battlefieldPanel.getScale();
		int cy = ((battlefieldPanel.getHeight() / 2) - battlefieldPanel.getDy())
				/ battlefieldPanel.getScale();
		int scale = battlefieldPanel.getScale() - value;
		if (scale >= MAX_SCALE) {
			scale = MAX_SCALE;
			zoomin.setEnabled(false);
		} else {
			zoomin.setEnabled(true);
		}
		if (scale <= 1) {
			scale = 1;
			zoomout.setEnabled(false);
		} else {
			zoomout.setEnabled(true);
		}
		int zoom = battlefieldPanel.getScale() - scale;
		battlefieldPanel.setScale(scale);
		battlefieldPanel.setDx(battlefieldPanel.getDx() + cx * zoom);
		battlefieldPanel.setDy(battlefieldPanel.getDy() + cy * zoom);
	}

	/**
	 * build the buttons panel
	 * 
	 */
	private void buildTextButtonsPanel() {
		textButtonsPanel.removeAll();
		textButtonsPanel.add(nextPhaseButton);
		textButtonsPanel.add(magic);
		textButtonsPanel.add(newOrders);
		if (!FB.isNet()) {
			textButtonsPanel.add(saveGame);
		}
		if (Battle.getInstance().isCampaignGame()) {
			textButtonsPanel.add(rout);
		} else {
			textButtonsPanel.add(quit);
		}
	}

	/**
	 * center the battlefield view on an army node
	 * 
	 * @param node
	 *            the army node
	 */
	public void centerOn(Object node) {
		Entity entity = null;
		if (node instanceof Army) {
			entity = ((Army) node).getWarchief();
		} else if (node instanceof Command) {
			entity = ((Command) node).getLeader();
		} else {
			entity = (Entity) node;
		}
		if (entity != null) {
			battlefieldPanel.centerOn(entity.getX(), entity.getY());
		}
	}

	/**
	 * start frame animation
	 * 
	 */
	public void start() {
		buildTextButtonsPanel();
		battlefieldPanel.start();
		int scale = battlefieldPanel.getScale();
		zoomin.setEnabled(scale != MAX_SCALE);
		zoomout.setEnabled(scale != 1);

	}

	/**
	 * stop frame animation
	 * 
	 */
	public void stop() {
		battlefieldPanel.stop();
	}

	/**
	 * Get the battlefieldPanel
	 * 
	 * @return the battlefieldPanel.
	 */
	public BattleFieldPanel getBattlefieldPanel() {
		return this.battlefieldPanel;
	}

	/**
	 * @return Returns the minimap.
	 */
	public MiniBattleFieldMapPanel getMinimap() {
		return minimap;
	}

	/**
	 * @return Returns the nextPhaseButton.
	 */
	public FBButton getNextPhaseButton() {
		return nextPhaseButton;
	}

	/**
	 * build all action panels
	 * 
	 */
	public void buildActionPanels() {

		if (unitMovementPanel == null) {
			unitMovementPanel = new UnitManoeuvrePanel();
		}
		if (expandPanel == null) {
			expandPanel = new ExpandPanel();
		}
		if (reducePanel == null) {
			reducePanel = new ReducePanel();
		}
		if (indvdMovementPanel == null) {
			indvdMovementPanel = new IndividualManoeuvrePanel();
		}
		if (orderpanel == null) {
			orderpanel = new OrderPanel();
		}
		if (wizardPanel == null) {
			wizardPanel = new WizardSpellPanel();
		}
		if (drdPanel == null) {
			drdPanel = new DruidSpellPanel();
		}
		if (priestPanel == null) {
			priestPanel = new PriestSpellPanel();
		}

		if (phasepanel == null) {
			phasepanel = new PhasePanel();
		}
		if (spellPanel == null) {
			spellPanel = new SpellPanel();
		}
	}

	/**
	 * change current phase
	 * 
	 * @param phase
	 *            the new phase
	 */
	public void setPhase(Phase phase) {
		BattleStatePanel.getInstance().setPhase(phase);

		if (phase.isWithPhasePanel()) {
			BattleStatePanel.getInstance().setHelp(
					Resources.getString(phase.getName() + "help"));

			phasepanel.setPhase(phase);

			setActionPanel(phasepanel);
		}
	}

	/**
	 * display movement panel for specified entity
	 * 
	 * @param e
	 *            the entity
	 */
	public void setMovementPanel(Entity e) {
		if (e != null) {
			if (e.isUnit()) {
				unitMovementPanel.setUnit((Unit) e);
				setActionPanel(unitMovementPanel);
			} else {
				indvdMovementPanel.setIndividual((Individual) e);
				setActionPanel(indvdMovementPanel);
			}
		} else {
			clearActionPanel();
		}
	}

	/**
	 * display expand panel for specified unit
	 * 
	 * @param u
	 *            the unit
	 */
	public void setExpandPanel(Unit u) {
		expandPanel.setUnit(u);
		setActionPanel(expandPanel);
	}

	/**
	 * display reduce panel for specified unit
	 * 
	 * @param u
	 *            the unit
	 */
	public void setReducePanel(Unit u) {
		reducePanel.setUnit(u);
		setActionPanel(reducePanel);
	}

	/**
	 * display order choice panel
	 * 
	 * @param op
	 *            the order phase
	 */
	public void setOrderPanel(PlayerOrderPhase op) {
		orderpanel.setPhase(op);
		setActionPanel(orderpanel);
	}

	/**
	 * display magic panel
	 * 
	 * @param mp
	 *            the magic phase
	 */
	public void setMagPanel(PlayerMagicPhase mp) {
		if (mp.getActor() instanceof Wizard) {
			wizardPanel.enableAllButtons();
			setActionPanel(wizardPanel);
		} else if (mp.getActor() instanceof Druid) {
			drdPanel.enableAllButtons();
			setActionPanel(drdPanel);
		} else if (mp.getActor() instanceof Priest) {
			priestPanel.enableAllButtons();
			setActionPanel(priestPanel);
		}
	}

	/**
	 * @return Returns the indvdMovementPanel.
	 */
	public IndividualManoeuvrePanel getIndvdMovementPanel() {
		return indvdMovementPanel;
	}

	/**
	 * @return Returns the spellPanel.
	 */
	public SpellPanel getSpellPanel() {
		return spellPanel;
	}

	/**
	 * @return Returns the unitMovementPanel.
	 */
	public UnitManoeuvrePanel getUnitMovementPanel() {
		return unitMovementPanel;
	}

}