/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames.popups;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.ComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBComboBox;
import fb.main.FB;
import fb.util.ComboArrayListModel;
import fb.util.FileArrayListModel;
import fb.util.Resources;
/**
 * an internal frame used to choose a file in a directory
 * @author Farfpeuf
 */
public class FilePopup extends JInternalFrame {
	/**
	 * chosen path
	 */
	private String pathToReturn = null;
	/**
	 * the directory to scan
	 */
	private String directoryPath;
	/**
	 * the file combo box
	 */
	private JComboBox fileCombo;
	/**
	 * constructor
	 * 
	 * build and assemble elements
	 * 
	 * @param dirPath the directory to scan
	 * @param key the title key
	 * @param labelKey the message key
	 * @param saveAsMode is frame in save as mode
	 */
	public FilePopup(String dirPath, String key, String labelKey,
			boolean saveAsMode) {
		super(Resources.getString(key), false, false, false, false);
		directoryPath = dirPath;
		ComboBoxModel model=new ComboArrayListModel(FileArrayListModel.getInstance(dirPath));
		fileCombo = new FBComboBox(model);
		fileCombo.setEditable(saveAsMode);
		if (fileCombo.getItemCount() != 0) {
			fileCombo.setSelectedIndex(0);
		}
		FBButton ok = new FBButton(Resources.getString("OK"));
		FBButton cancel = new FBButton(Resources.getString("Cancel"));
		JLabel nameLabel = new JLabel(Resources.getString(labelKey));
		JPanel cont = new JPanel();
		cont.setLayout(new GridLayout(2, 2));
		cont.add(nameLabel);
		cont.add(fileCombo);
		cont.add(ok);
		cont.add(cancel);
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev){
				String selectedItem = (String) fileCombo.getSelectedItem();
				if (selectedItem != null && !selectedItem.equals("")) {
					pathToReturn = directoryPath + File.separator+ selectedItem;
					closeFrame();
				}
			}
		});
		cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				closeFrame();
			}
		});
		getContentPane().add(cont);
		setBounds((int)(FB.getXres()*400), (int)(FB.getYres()*130), 260, 80);
		validate();
		
	}
	/**
	 * get a file
	 * @return the file path
	 */
	public synchronized String getFile() {
		toFront();
		show();
		try {
			wait();
		} catch (InterruptedException e) {
			System.out.println("interrupted");
		}
		return pathToReturn;
	}
	/**
	 * close the frame
	 *
	 */
	private synchronized void closeFrame() {
		dispose();
		try {
			notify();
		} catch (IllegalMonitorStateException e) {
			System.out.println("illegal monitor state");
			e.printStackTrace();
		}
	}
}