/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames.popups;

import java.awt.GridLayout;

import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.main.FB;
import fb.util.Resources;

/**
 * a frame to display a multiple text fields
 * 
 * @author Farfpeuf
 */
public class MutlipleFieldPopup extends JInternalFrame {
	/**
	 * entered values
	 */
	private String[] stringsToReturn = null;

	/**
	 * text fields
	 */
	private JTextField[] fields;

	/**
	 * constructor
	 * 
	 * build and assemble elements
	 * 
	 * @param key
	 *            title key
	 * @param labelKeys
	 *            fields labels keys
	 * @param initialValue
	 *            fields initial values
	 */
	public MutlipleFieldPopup(String key, String[] labelKeys,
			String[] initialValue) {
		super(Resources.getString(key), false, false, false, false);
		int nbItems = labelKeys.length;
		JLabel nameLabel;
		JPanel cont = new JPanel();
		fields = new JTextField[nbItems];
		stringsToReturn = new String[nbItems];
		cont.setLayout(new GridLayout(nbItems + 1, 2));
		for (int i = 0; i < nbItems; i++) {
			nameLabel = new JLabel(Resources.getString(labelKeys[i]));
			nameLabel.setHorizontalAlignment(SwingConstants.CENTER);
			fields[i] = new JTextField(initialValue[i]);
			fields[i].setHorizontalAlignment(SwingConstants.CENTER);
			cont.add(nameLabel);
			cont.add(fields[i]);
		}
		FBButton ok = new FBButton(Resources.getString("OK"));
		FBButton cancel = new FBButton(Resources.getString("Cancel"));
		cont.add(ok);
		cont.add(cancel);

		ok.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				int nbItems = fields.length;
				stringsToReturn = new String[nbItems];
				for (int i = 0; i < nbItems; i++) {
					stringsToReturn[i] = fields[i].getText();
				}
				closeFrame();

			}
		});

		cancel.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				stringsToReturn = null;
				closeFrame();
			}
		});

		getContentPane().add(cont);

		setBounds((int) (300 * FB.getXres()), (int) (100 * FB.getYres()),
				(int) Math.max(80, (200 * FB.getXres())), (int) Math.max(80,
						((40 + 20 * nbItems) * FB.getYres())));

		validate();

	}

	/**
	 * get the results
	 * 
	 * @return a string array of text field values
	 */
	public synchronized String[] getStrings() {

		toFront();
		show();
		try {
			wait();
		} catch (InterruptedException e) {
			System.out.println("interrupted");
		}
		return stringsToReturn;
	}

	/**
	 * close this frame
	 * 
	 */
	private synchronized void closeFrame() {
		FB.getInstance().requestFocus();
		dispose();
		try {
			notify();

		} catch (IllegalMonitorStateException e) {
			System.out.println("illegal monitor state");
			e.printStackTrace();
		}
	}
}
