/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ToolTipManager;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import fb.army.entities.Entity;
import fb.army.hierarchy.AllyCommand;
import fb.army.hierarchy.Army;
import fb.army.hierarchy.Command;
import fb.army.hierarchy.MercenaryCommand;
import fb.army.hierarchy.WarchiefCommand;
import fb.gui.frames.popups.MutlipleFieldPopup;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.CatalogCell;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBComboBox;
import fb.gui.widgets.FBPopUpButton;
import fb.gui.widgets.RecruitListener;
import fb.gui.widgets.armytreetable.ArmyTreeModel;
import fb.gui.widgets.armytreetable.TreeTable;
import fb.gui.widgets.armytreetable.TreeTableModelAdapter;
import fb.main.Catalog;
import fb.main.FB;
import fb.util.FBColors;
import fb.util.FileTools;
import fb.util.PercentageLayout;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * army editor main panel
 * 
 * @author farfpeuf
 * 
 */
public class ArmyEditorPanel extends JPanel {

	/**
	 * singleton instance
	 */
	private static ArmyEditorPanel instance;

	/**
	 * current army tree representation
	 */
	private TreeTable armyTreeTable;

	/**
	 * sub panels
	 */
	private JPanel availableEntitiesPanel, recruitPanel, armyPanel,
			listBorderPanel, armyBorderPanel;

	/**
	 * panel containing army tree
	 */
	private TreeTablePanel armyTreePanel;

	/**
	 * current race header (combo or label)
	 */
	private JComponent currentListHeader = null;

	/**
	 * menu to add a new command to the army
	 */
	private FBPopUpButton newCommandMenu = new FBPopUpButton(Resources
			.getString("Newcommand"));

	/**
	 * sub menus
	 */
	private JMenuItem newCommand = new JMenuItem(Resources
			.getString("Newcommand")), newAllied = new JMenuItem(Resources
			.getString("Alliedcommand")), newMercenary = new JMenuItem(
			Resources.getString("Mercenarycommand"));

	/**
	 * selected entities
	 */
	private Vector treeSelectedGroups, treeSelectedEntities;

	/**
	 * available entities
	 */
	private JList availableEntitiesList;

	/**
	 * current army
	 */
	private Army army;

	/**
	 * additional panel for entities recruitment
	 */
	private EntityRecruitPanel entityRecrutPanel;

	/**
	 * current selected command
	 */
	private Command treeSelectedCommand;

	/**
	 * current selected entity
	 */
	private Entity treeSelectedEntity;

	/**
	 * combo to choose race
	 */
	private FBComboBox raceSelector = new FBComboBox();

	/**
	 * label displaying current race of available entities list
	 */
	private JLabel raceLabel = new JLabel("", 0);

	/**
	 * label for budget
	 */
	private JLabel budgetLabel = new JLabel(Resources.getString("budget"));

	/**
	 * label displaying army current budget
	 */
	private JLabel budgetLab = new JLabel();

	/**
	 * button for adding and removing entities
	 */
	private FBButton addEntity = new FBButton(Resources.getString("Addentity")),
			remove = new FBButton(Resources.getString("Removeentity"));

	/**
	 * army initial budget
	 */
	private int initialBudget;

	/**
	 * constructor build and assemble panel elements and hook up events
	 */
	private ArmyEditorPanel() {
		entityRecrutPanel = new EntityRecruitPanel();

		availableEntitiesList = new JList();
		availableEntitiesList.setCellRenderer(new CatalogCell());
		ToolTipManager.sharedInstance()
				.registerComponent(availableEntitiesList);
		JScrollPane scrolljlist = new JScrollPane(availableEntitiesList);

		scrolljlist.setOpaque(false);
		scrolljlist.getViewport().setOpaque(false);
		scrolljlist.setBorder(null);

		raceLabel.setOpaque(true);
		raceLabel.setVisible(false);
		raceSelector.setVisible(false);
		listBorderPanel = new JPanel();
		listBorderPanel.setBorder(BorderFactory
				.createLineBorder(FBColors.darkBorder));
		listBorderPanel.setLayout(new GridLayout(1, 1));
		availableEntitiesPanel = new JPanel();
		listBorderPanel.add(availableEntitiesPanel);
		availableEntitiesPanel.setOpaque(false);
		availableEntitiesPanel.setLayout(new PercentageLayout());
		availableEntitiesPanel.add(raceSelector, new PercentageLayout.Constraint(
				1, 0, 98, 16));
		availableEntitiesPanel.add(raceLabel, new PercentageLayout.Constraint(0,
				0, 100, 16));
		availableEntitiesPanel.add(scrolljlist, new PercentageLayout.Constraint(1,
				16, 99, 84));
		recruitPanel = new JPanel();
		armyPanel = new JPanel();
		armyBorderPanel = new JPanel();

		armyBorderPanel.setBorder(BorderFactory
				.createLineBorder(FBColors.darkBorder));
		armyBorderPanel.setLayout(new GridLayout(1, 1));
		armyBorderPanel.add(armyPanel);
		armyPanel.setOpaque(false);
		recruitPanel.setOpaque(false);
		recruitPanel.setLayout(new PercentageLayout());
		setLayout(new PercentageLayout());
		recruitPanel.add(entityRecrutPanel, new PercentageLayout.Constraint(48, 8,
				44, 92));
		recruitPanel.add(listBorderPanel, new PercentageLayout.Constraint(8, 8,
				30, 92));
		setOpaque(false);

		add(addEntity, new PercentageLayout.Constraint(35, 42, 12, 4));
		add(remove, new PercentageLayout.Constraint(50, 42, 12, 4));
		add(recruitPanel, new PercentageLayout.Constraint(0, 0, 100, 38));
		add(armyBorderPanel, new PercentageLayout.Constraint(0, 50, 100, 50));

		addEntity.setEnabled(false);
		remove.setEnabled(false);
		validate();
		newCommandMenu.add(newCommand);
		newCommandMenu.add(newAllied);
		newCommandMenu.add(newMercenary);

		availableEntitiesList.setToolTipText(Resources.getString("zone1.tt"));
		armyBorderPanel.setToolTipText(Resources.getString("zone3.tt"));
		addEntity.setToolTipText(Resources.getString("recruitbutton.tt"));
		remove.setToolTipText(Resources.getString("firebutton.tt"));
		newCommandMenu.setToolTipText(Resources.getString("group.tt"));
		newCommand.setToolTipText(Resources.getString("standardcommand.tt"));
		newAllied.setToolTipText(Resources.getString("alliedcommand.tt"));
		newMercenary.setToolTipText(Resources.getString("mercenarycommand.tt"));
		armyPanel.setLayout(new PercentageLayout());
		armyTreePanel = new TreeTablePanel(army, false);
		armyTreePanel.getTreeTable().setColumnWidths(
				new int[] { 300, 80, 100, 80 });
		armyTreeTable = armyTreePanel.getTreeTable();

		armyPanel.setBackground(FBColors.background);
		armyPanel
				.add(newCommandMenu, new PercentageLayout.Constraint(0, 0, 18, 8));
		armyPanel.add(budgetLabel, new PercentageLayout.Constraint(80, 0, 15, 8));
		armyPanel.add(budgetLab, new PercentageLayout.Constraint(95, 0, 5, 8));
		armyPanel.add(armyTreePanel,
				new PercentageLayout.Constraint(0, 8, 100, 92));

		validate();
		repaint();
		hookupEvents();
	}

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static synchronized ArmyEditorPanel getInstance() {
		if (null == instance) {
			instance = new ArmyEditorPanel();
		}
		return instance;
	}

	/**
	 * delete singleton instance to free memory
	 * 
	 */
	public static void freeInstance() {
		if (null != instance) {
			instance = null;
		}
	}

	/**
	 * handle selection change in available entities list
	 * 
	 * @param e
	 *            the ListSelectionEvent
	 */
	public void availableEntitiesSelectionChanged(ListSelectionEvent e) {

		Entity entity = (Entity) availableEntitiesList.getSelectedValue();
		if (entity != null) {

			setCurrentEntity(entity);
			listBorderPanel.setBorder(BorderFactory
					.createLineBorder(FBColors.lightBorder));
			armyBorderPanel.setBorder(BorderFactory
					.createLineBorder(FBColors.darkBorder));
			manageRecruitButton();
			armyTreeTable.getSelectionModel().clearSelection();
			if (treeSelectedCommand != null) {
				selectNode(treeSelectedCommand);
			}

		}
	}

	/**
	 * disable or enable "add entity" button according to selected entity
	 * 
	 */
	public void manageRecruitButton() {
		Entity entity = (Entity) availableEntitiesList.getSelectedValue();
		addEntity.setEnabled(canRecruit(entity));
	}

	/**
	 * test if selected entity can be added to the army
	 * 
	 * @param entity
	 *            the entity
	 * @return true if selected entity can be added to the army
	 */
	public boolean canRecruit(Entity entity) {

		if (entity == null || treeSelectedCommand == null) {
			return false;
		}
		return entity.isEnabled()
				&& (initialBudget - (army.getCost() + entity.getCost()) >= 0);
	}

	/**
	 * handle selcection change in army tree
	 * 
	 * @param e
	 *            the ListSelectionEvent
	 */
	public void armyTreeSelectionChanged(ListSelectionEvent e) {
		ListSelectionModel model = (ListSelectionModel) e.getSource();
		int iMin = model.getMinSelectionIndex();
		int iMax = model.getMaxSelectionIndex();

		if ((iMin == -1) || (iMax == -1)) {
			treeSelectedEntities.clear();
			treeSelectedGroups.clear();
			treeSelectedEntity = null;
			return;
		}

		TreeTableModelAdapter ad = (TreeTableModelAdapter) armyTreeTable
				.getModel();

		List selectedNodes = new ArrayList();
		for (int i = iMin; i <= iMax; i++) {
			if (model.isSelectedIndex(i)) {
				selectedNodes.add(ad.nodeForRow(i));
			}

		}

		Object leadNode = ad.nodeForRow(model.getLeadSelectionIndex());

		if (leadNode != null) {

			Command selectedGroup = null;
			treeSelectedEntity = null;

			TreeNode node;
			treeSelectedEntities = new Vector();
			treeSelectedGroups = new Vector();
			boolean removeEnabled = false;

			if (leadNode instanceof Entity) {
				selectedGroup = ((Entity) leadNode).getCommand();
				treeSelectedEntity = (Entity) leadNode;
			} else if (leadNode instanceof Command) {
				selectedGroup = (Command) leadNode;
			}

			for (int i = 0; i < selectedNodes.size(); i++) {
				node = (TreeNode) selectedNodes.get(i);
				if (node instanceof Entity) {
					treeSelectedEntities.add(node);
					removeEnabled = true;
				} else if (node instanceof Command) {
					treeSelectedGroups.add(node);
					if (!(node instanceof WarchiefCommand)) {
						removeEnabled = true;
					}
				}
			}

			setSelectedCommand(selectedGroup);
			remove.setEnabled(removeEnabled);
			manageRecruitButton();
			if (null != treeSelectedEntity) {
				availableEntitiesList.getSelectionModel().clearSelection();
				setCurrentEntity(treeSelectedEntity);
				armyBorderPanel.setBorder(BorderFactory
						.createLineBorder(FBColors.lightBorder));
				listBorderPanel.setBorder(BorderFactory
						.createLineBorder(FBColors.darkBorder));
			}
		}

	}

	/**
	 * @return the army tree model
	 */

	public ArmyTreeModel getArmyTreeModel() {
		TreeTableModelAdapter ad = (TreeTableModelAdapter) armyTreeTable
				.getModel();
		return (ArmyTreeModel) ad.getTreeTableModel();
	}

	/**
	 * refresh army tree
	 * 
	 */
	private void updateTree() {
		if (treeSelectedEntity != null) {
			getArmyTreeModel().nodeChanged(treeSelectedEntity);
			getArmyTreeModel().nodeChanged(treeSelectedEntity.getCommand());

			((TreeTableModelAdapter) armyTreeTable.getModel())
					.fireTableRowsUpdated(0, 0);
		}
	}

	/**
	 * change current displayed entity
	 * 
	 * @param e
	 *            the new entity
	 */
	public void setCurrentEntity(Entity e) {
		if (e != null) {
			entityRecrutPanel.setEntity(e);
			entityRecrutPanel.validate();
			entityRecrutPanel.repaint();
			entityRecrutPanel.setVisible(true);
		} else {
			entityRecrutPanel.setVisible(false);
		}
	}

	/**
	 * change current selected command
	 * 
	 * @param selectedgroup
	 *            the new command
	 */
	public void setSelectedCommand(Command selectedgroup) {
		if (selectedgroup != this.treeSelectedCommand) {
			this.treeSelectedCommand = selectedgroup;
			if (null != selectedgroup) {
				String race = army.getRace();
				if (null != currentListHeader) {
					currentListHeader.setVisible(false);
				}
				if (selectedgroup instanceof MercenaryCommand
						|| selectedgroup instanceof AllyCommand) {
					String[] catalog = selectedgroup.getAvailableRaces();
					raceSelector.setModel(new DefaultComboBoxModel(catalog));
					currentListHeader = raceSelector;
					raceSelector.setSelectedIndex(0);
				} else {
					raceLabel.setText(Resources.getString("ae" + race));
					currentListHeader = raceLabel;
					displayAvailableEntities(race);
				}
				currentListHeader.setVisible(true);
				availableEntitiesPanel.validate();
				availableEntitiesPanel.repaint();
			}
		}
	}

	/**
	 * display Available Entities for specified race
	 * 
	 * @param race
	 *            the entities race
	 */
	public void displayAvailableEntities(String race) {
		Vector availableEntities = Catalog.getAvailableEntities(race);

		availableEntitiesList.setListData(availableEntities);
		availableEntitiesPanel.validate();
		availableEntitiesPanel.repaint();
	}

	/**
	 * load an army file
	 * 
	 * @param file
	 *            the file
	 */
	public void openArmy(String file) {
		Reader input_file = null;
		StringBuffer ligne;
		String lineread;
		Command group = null;
		BufferedReader br;
		Army army;
		char ligneType;
		try {
			input_file = FileTools.getReader(file);
			br = new BufferedReader(input_file);
			ligne = new StringBuffer(br.readLine());
			army = new Army("", file);
			army.loadString(ligne);
			lineread = br.readLine();
			if (lineread != null) {
				ligne = new StringBuffer(lineread);
			} else {
				ligne = null;
			}
			while (ligne != null) {
				ligneType = StringTools.readBufferChar(ligne);
				if (ligneType == 'w') {
					group = new WarchiefCommand();
					group.loadString(ligne);
					army.addCommand(group);
				} else if (ligneType == 'a') {
					group = new AllyCommand();
					group.loadString(ligne);
					army.addCommand(group);
				} else if (ligneType == 'g') {
					group = new Command();
					group.loadString(ligne);
					army.addCommand(group);
				} else if (ligneType == 'y') {
					group = new MercenaryCommand();
					group.loadString(ligne);
					army.addCommand(group);
				} else if (ligneType == 'e') {
					int linenumber = StringTools.readBufferInt(ligne);
					Entity entity = (Entity) ((Entity) Catalog.getFullCatalog()
							.get(linenumber)).clone();
					entity.loadString(ligne);
					entity.setLineNumber(linenumber);
					group.addEntity(entity);
				}
				lineread = br.readLine();
				if (lineread != null) {
					ligne = new StringBuffer(lineread);
				} else {
					ligne = null;
				}
			} // while
			br.close();
			input_file.close();
			setArmy(army);
		} // try
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * add a command to current army
	 * 
	 * @param command
	 */
	private void addCommand(Command command) {
		getArmyTreeModel().insertNodeInto(command, army, army.getChildCount());
		TreePath path = new TreePath(new Object[] { army, command });
		armyTreeTable.getTree().expandPath(path);
		armyTreeTable.getTree().setSelectionPath(path);
	}

	/**
	 * remove selected commands or entities
	 * 
	 */
	private void remove() {
		if (!treeSelectedEntities.isEmpty() || !treeSelectedGroups.isEmpty()) {
			MutableTreeNode node = null;
			TreeNode nodeToSelect = null;
			Vector nodesToRemove = new Vector();
			Command group;
			Entity entity;

			for (int i = 0; i < treeSelectedGroups.size(); i++) {
				group = (Command) treeSelectedGroups.get(i);
				if (!(group instanceof WarchiefCommand)) {
					nodesToRemove.add(group);
					if (group instanceof MercenaryCommand) {
						newMercenary.setEnabled(true);
					} else if (group instanceof AllyCommand) {
						newAllied.setEnabled(true);
					}
				}
			}
			for (int i = 0; i < treeSelectedEntities.size(); i++) {
				entity = (Entity) treeSelectedEntities.get(i);
				if (!nodesToRemove.contains(entity.getParent())) {
					nodesToRemove.add(entity);
				}
			}
			for (int i = 0; i < nodesToRemove.size(); i++) {
				node = (MutableTreeNode) nodesToRemove.get(i);
				if (i == nodesToRemove.size() - 1) { // last node
					nodeToSelect = getNextNode(node);
				}
				getArmyTreeModel().removeNodeFromParent(node);
				if (node instanceof Entity) {
					getArmyTreeModel().nodeChanged(node.getParent());
				}
			}
			selectNode(nodeToSelect);
			updateBudget();
		}
	}

	/**
	 * select a tree node in the army tree
	 * 
	 * @param node
	 *            the node to select
	 */
	private void selectNode(TreeNode node) {
		if (node != null) {
			if (node instanceof Entity) {
				armyTreeTable.getTreeSelectionModel().setSelectionPath(
						new TreePath(new Object[] { army, node.getParent(),
								node }));
			} else {
				armyTreeTable.getTreeSelectionModel().setSelectionPath(
						new TreePath(new Object[] { army, node }));
			}
		}
	}

	/**
	 * get node after this one
	 * 
	 * @param node
	 *            the node
	 * @return the node after
	 */
	private TreeNode getNextNode(TreeNode node) {
		TreeNode nextSelectedNode;
		int index = node.getParent().getIndex(node);
		if (index == 0) {
			nextSelectedNode = node.getParent();
		} else if (index == node.getParent().getChildCount() - 1) {
			nextSelectedNode = node.getParent().getChildAt(index - 1);
		} else {
			nextSelectedNode = node.getParent().getChildAt(index + 1);
		}
		return nextSelectedNode;
	}

	/**
	 * add an entity to the army
	 * 
	 */
	private void addEntity() {
		if (availableEntitiesList.getSelectedValue() != null) {
			MutableTreeNode node = (MutableTreeNode) armyTreeTable
					.getTreeSelectionModel().getSelectionPath()
					.getLastPathComponent();
			Entity clone = (Entity) ((Entity) availableEntitiesList
					.getSelectedValue()).clone();

			Command group = null;
			if (node instanceof Entity) {
				group = ((Entity) node).getCommand();
			} else if (node instanceof Command) {
				group = (Command) node;
			}

			getArmyTreeModel().insertNodeInto(clone, group,
					group.getChildCount());
			getArmyTreeModel().nodeChanged(group);
			updateBudget();
		}
	}

	/**
	 * update army budget
	 * 
	 */
	private void updateBudget() {
		int budget = initialBudget - army.getCost();
		budgetLab.setText("" + budget);
	}

	/**
	 * register all widgets events
	 * 
	 */
	private void hookupEvents() {

		entityRecrutPanel.addRecrutListener(new RecruitListener() {
			public void entityChanged() {
				updateTree();
				manageRecruitButton();
				updateBudget();
			}
		});
		raceSelector.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				String race = (String) ((JComboBox) ev.getSource())
						.getSelectedItem();
				displayAvailableEntities(Resources.getString(StringTools
						.replaceAll(race, " ", "")));
			}
		});
		armyTreeTable.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent e) {
						armyTreeSelectionChanged(e);
					}
				});
		availableEntitiesList
				.addListSelectionListener(new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent e) {
						availableEntitiesSelectionChanged(e);
					} // mouse value changed
				});
		// recruter
		addEntity.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addEntity();
			}
		});
		// virer
		remove.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				remove();
			}
		});
		// newcommand
		newCommand.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				MutlipleFieldPopup joinBox = new MutlipleFieldPopup(Resources
						.getString("Newcommand"), new String[] { "nodename" },
						new String[] { Resources.getString("Othercommand") });
				FB.getInstance().getContentPane().add(joinBox, 0);
				String[] params = joinBox.getStrings();
				if (null != params && params[0].length() > 0) {

					Command group = new Command();
					group.setName(params[0]);
					addCommand(group);
				}
			}
		});
		// newallied
		newAllied.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				MutlipleFieldPopup joinBox = new MutlipleFieldPopup(Resources
						.getString("Alliedcommand"),
						new String[] { "nodename" }, new String[] { Resources
								.getString("Alliedcommand") });
				FB.getInstance().getContentPane().add(joinBox, 0);
				String[] params = joinBox.getStrings();
				if (null != params && params[0].length() > 0) {
					Command group = new AllyCommand();
					group.setName(params[0]);
					addCommand(group);
					newAllied.setEnabled(false);
				}
			}
		});
		// newmercenary
		newMercenary.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				MutlipleFieldPopup joinBox = new MutlipleFieldPopup(Resources
						.getString("Mercenarycommand"),
						new String[] { "nodename" }, new String[] { Resources
								.getString("Mercenarycommand") });
				FB.getInstance().getContentPane().add(joinBox, 0);
				String[] params = joinBox.getStrings();
				if (null != params && params[0].length() > 0) {
					Command group = new MercenaryCommand();
					group.setName(params[0]);
					addCommand(group);
					newMercenary.setEnabled(false);
				}
			}
		});

	} // hookupevents

	/**
	 * Sets the army.
	 * 
	 * @param army
	 *            The army to set
	 */
	public void setArmy(Army army) {
		this.army = army;
		boolean noAllied = true, noMercenary = true;
		if (army.getCommands().isEmpty()) {
			Command wg = new WarchiefCommand();
			wg.setName(Resources.getString("Warchiefcommand"));
			army.recruitCommand(wg);
		} else {
			Iterator iter = army.getAllGroups().iterator();
			Command group;
			while (iter.hasNext()) {
				group = (Command) iter.next();
				if (group instanceof MercenaryCommand) {
					noMercenary = false;
				} else if (group instanceof AllyCommand) {
					noAllied = false;
				}
			}
		}
		newMercenary.setEnabled(noMercenary);
		newAllied.setEnabled(noAllied);
		armyTreePanel.setRoot(army);

		if (!army.getCommands().isEmpty()) {
			Command group = (Command) army.getCommands().get(0);
			if (!group.getActiveEntities().isEmpty()) {
				Entity entity = (Entity) group.getActiveEntities().get(0);
				armyTreeTable.getTreeSelectionModel().setSelectionPath(
						new TreePath(new Object[] { army, group, entity }));
			} else {
				armyTreeTable.getTreeSelectionModel().setSelectionPath(
						new TreePath(new Object[] { army, group }));
				if (availableEntitiesList.getModel().getSize() > 0) {
					availableEntitiesList.setSelectedIndex(0);
				}
			}
		} else {
			armyTreeTable.getTreeSelectionModel().setSelectionPath(
					new TreePath(new Object[] { army }));
		}

		updateBudget();

	}

	/**
	 * set the army initial budget
	 * 
	 * @param budget
	 *            the budget
	 */
	public void setInitialBudget(int budget) {
		this.initialBudget = budget;
		budgetLab.setText("" + budget);
	}

	/**
	 * Returns the arm.
	 * 
	 * @return Army
	 */
	public Army getArmy() {
		return army;
	}

	/**
	 * @return Returns the initialBudget.
	 */
	public int getInitialBudget() {
		return initialBudget;
	}

}