/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fb.army.entities.Entity;
import fb.util.PercentageLayout;
import fb.util.Resources;
/**
 * panel to display entity characteristics
 * @author farfpeuf
 *
 */
public class EntityPanel extends  JPanel {
	/**
	 * cells to display entity characteristics
	 */
	public JLabel celllevel, celltype, cellarmor, cellweapon, cellbadlight,
			title, photo, cellisterrible, cellstrength, cellresistance,
			cellworth, cellmovement, cellpdv;
	/**
	 * sub panels
	 */
	public JPanel leftPanel, middlePanel, rightPanel, photoPanel, titlePanel,icoPanel;
	/**
	 * icons
	 */
	protected static ImageIcon 
	terribleIcon = Resources.getMinicon("spectre"),
	dayIcon = Resources.getSign("day"),
	nightIcon = Resources.getSign("night"),
	dayNightIcon = Resources.getSign("daynight");
	/**
	 * current entity displayed
	 */
	protected Entity entity;
	/**
	 * constructor
	 * build and assemble panel elements
	 */
	public EntityPanel() {
		super();
		leftPanel = new JPanel();
		middlePanel = new JPanel();
		rightPanel = new JPanel();
		photoPanel = new JPanel();
		titlePanel = new JPanel();
		icoPanel = new JPanel();
		celllevel = new JLabel();
		celltype = new JLabel();
		cellarmor = new JLabel();
		cellweapon = new JLabel();
		cellbadlight = new JLabel();
		cellisterrible = new JLabel();
		cellstrength = new JLabel();
		cellresistance = new JLabel();
		cellworth = new JLabel();
		cellmovement = new JLabel();
		cellpdv = new JLabel();
		title = new JLabel();
		photo = new JLabel();
		
		setLayout(new PercentageLayout());
		leftPanel.setLayout(new GridLayout(4, 1,2,2));
		middlePanel.setLayout(new GridLayout(4, 1,2,2));
		rightPanel.setLayout(new GridLayout(4, 1,2,2));
		photoPanel.setLayout(new GridBagLayout());
		titlePanel.setLayout(new GridLayout(1, 1,2,2));		
		icoPanel.setLayout(new GridBagLayout());		
		
		title.setHorizontalAlignment(0);
		leftPanel.add(celllevel);
		leftPanel.add(celltype);
		leftPanel.add(cellweapon);
		leftPanel.add(cellarmor);
		middlePanel.add(cellworth);
		middlePanel.add(cellstrength);
		middlePanel.add(cellresistance);
		middlePanel.add(cellmovement);
		rightPanel.add(cellpdv);
		rightPanel.add(icoPanel);
		icoPanel.add(cellbadlight);
		icoPanel.add(cellisterrible);
		icoPanel.add(Box.createGlue(), new GridBagConstraints(2, 0, 1, 1,
				1.0, 0.0, GridBagConstraints.CENTER,
				GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
		photoPanel.add(photo);
        
        titlePanel.setLayout(new GridBagLayout());
        GridBagConstraints gc=new GridBagConstraints();
        gc.weightx=1;
        gc.anchor=GridBagConstraints.CENTER;
		titlePanel.add(title,gc );

		add(
				photoPanel,
				new PercentageLayout.Constraint(2, 10, 20, 80));
			add(
				titlePanel,
				new PercentageLayout.Constraint(24, 0, 76, 20));
			add(
				leftPanel,
				new PercentageLayout.Constraint(24, 20, 28, 80));
			add(
				middlePanel,
				new PercentageLayout.Constraint(52, 20, 26, 80));
			add(
				rightPanel,
				new PercentageLayout.Constraint(78, 20, 22, 80));
			

		validate();
        
        
        
        celllevel.setToolTipText(Resources.getString("level.tt"));
        celltype.setToolTipText(Resources.getString("discipline.tt"));
        cellarmor.setToolTipText(Resources.getString("armor.tt"));
        cellweapon.setToolTipText(Resources.getString("weapon.tt"));
        cellbadlight.setToolTipText(Resources.getString("daynight.tt"));
        cellisterrible.setToolTipText(Resources.getString("frightening.tt"));
        cellstrength.setToolTipText(Resources.getString("force.tt"));
        cellresistance.setToolTipText(Resources.getString("constitution.tt"));
        cellworth.setToolTipText(Resources.getString("attack.tt"));
        cellmovement.setToolTipText(Resources.getString("move.tt"));
		cellpdv.setToolTipText(Resources.getString("pdv.tt"));

        
        
	}
	/**
	 * display characteristics for specified entity
	 * @param e the entity
	 */
	public void setEntity(Entity e) {

		this.entity = e;

		
		
		celllevel.setText(Resources.getString("Level") + " : " + e.levelString());
		celltype.setText(Resources.getString("Discipline") + " : " + e.typeString());
		if (e.hasShield()) {
			cellarmor.setText(Resources.getString("Armor") + " : " + e.armorString()
					+ "/" + Resources.getString("shield"));
		} else {
			cellarmor.setText(Resources.getString("Armor") + " : " + e.armorString());
		}
		cellweapon.setText(Resources.getString("Weapon") + " : " + e.weaponString());
		if (e.getBadLight() == 'd') {
			cellbadlight.setIcon(nightIcon);
		} else if (e.getBadLight() == 'n') {
			cellbadlight.setIcon(dayIcon);
		} else {
			cellbadlight.setIcon(dayNightIcon);
		}
		if (e.isTerrible()) {
			cellisterrible.setIcon(terribleIcon);;
		} else {
			cellisterrible.setIcon(null);
		}

		cellstrength.setText(Resources.getString("Strength") + " : "
				+ e.strengthString());
		cellresistance.setText(Resources.getString("Constitution") + " : "
				+ e.resistanceString());
		cellworth.setText(Resources.getString("Attacks") + " : " + e.worthString());


		title.setIcon(e.getMinicon());
		photo.setIcon(e.getPhoto());
	}

}