/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets.armytreetable;

import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;

import fb.army.hierarchy.HierarchyNode;
import fb.util.Resources;

/**
 * data model for amry tree table
 * 
 * @author farfpeuf
 */

public class ArmyTreeModel extends DefaultTreeModel implements TreeTableModel {
	/**
	 * table column names
	 */
	private String[] columnNames;

	/**
	 * table column types
	 */
	private Class[] columnClasses;

	/**
	 * tree-table used in army editor or in battle information table ?
	 */
	private boolean battleMode;

	/**
	 * constructor
	 * 
	 * @param battleMode
	 *            tree-table used in army editor or in battle information table ?
	 */
	public ArmyTreeModel(boolean battleMode) {
		super(null);
		this.battleMode = battleMode;

		if (battleMode) {
			columnNames = new String[] { Resources.getString("nodename"),
					Resources.getString("nodenumber"),
					Resources.getString("nodespecial") };

			columnClasses = new Class[] { TreeModel.class, Integer.class,
					MutableTreeNode.class };
		} else {
			columnNames = new String[] { Resources.getString("nodename"),
					Resources.getString("nodenumber"),
					Resources.getString("nodespecial"),
					Resources.getString("Cost") };
			columnClasses = new Class[] { TreeModel.class, Integer.class,
					MutableTreeNode.class, Integer.class };
		}

	}

	/**
	 * Indicates whether the the value for node <code>node</code>, at column
	 * number <code>column</code> is editable.
	 */
	public boolean isCellEditable(Object node, int column) {
		return getColumnClass(column) == TreeModel.class;
	}

	/**
	 * Sets the value for node <code>node</code>, at column number
	 * <code>column</code>.
	 */
	public void setValueAt(Object aValue, Object node, int column) {
	}

	/**
	 * get children of specified parent node
	 * 
	 * @param node
	 *            the parent node
	 * @return children node array
	 */
	protected Object[] getChildren(Object node) {
		HierarchyNode armyNode = ((HierarchyNode) node);
		Object[] result = new Object[armyNode.getChildCount()];
		for (int i = 0; i < result.length; i++) {
			result[i] = armyNode.getChildAt(i);
		}
		return result;
	}

	/**
	 * Returns the number of children of <code>parent</code>. Returns 0 if
	 * the node is a leaf or if it has no children. <code>parent</code> must
	 * be a node previously obtained from this data source.
	 * 
	 * @param parent
	 *            a node in the tree, obtained from this data source
	 * @return the number of children of the node <code>parent</code>
	 */
	public int getChildCount(Object node) {
		Object[] children = getChildren(node);
		return (children == null) ? 0 : children.length;
	}

	/**
	 * Returns the child of <code>parent</code> at index <code>index</code>
	 * in the parent's child array. <code>parent</code> must be a node
	 * previously obtained from this data source. This should not return
	 * <code>null</code> if <code>index</code> is a valid index for
	 * <code>parent</code> (that is <code>index >= 0 &&
	 * index < getChildCount(parent</code>)).
	 * 
	 * @param parent
	 *            a node in the tree, obtained from this data source
	 * @return the child of <code>parent</code> at index <code>index</code>
	 */
	public Object getChild(Object node, int i) {
		return getChildren(node)[i];
	}

	/**
	 * Returns <code>true</code> if <code>node</code> is a leaf. It is
	 * possible for this method to return <code>false</code> even if
	 * <code>node</code> has no children. A directory in a filesystem, for
	 * example, may contain no files; the node representing the directory is not
	 * a leaf, but it also has no children.
	 * 
	 * @param node
	 *            a node in the tree, obtained from this data source
	 * @return true if <code>node</code> is a leaf
	 */
	public boolean isLeaf(Object node) {
		return ((HierarchyNode) node).isLeaf();
	}

	/**
	 * Returns the number ofs availible column.
	 */
	public int getColumnCount() {
		return columnNames.length;
	}

	/**
	 * Returns the name for column number <code>column</code>.
	 */
	public String getColumnName(int column) {
		return columnNames[column];
	}

	/**
	 * Returns the type for column number <code>column</code>.
	 */
	public Class getColumnClass(int column) {
		return columnClasses[column];
	}

	/**
	 * Returns the value to be displayed for node <code>node</code>, at
	 * column number <code>column</code>.
	 */
	public Object getValueAt(Object node, int column) {
		if (node != null) {
			HierarchyNode armyNode = ((HierarchyNode) node);
			switch (column) {
			case 0:
				return armyNode;
			case 1:
				return armyNode.numberString();
			case 2:
				return armyNode;
			case 3:
				return new Integer(armyNode.getCost());
			}
		}
		return null;
	}
}
