/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import java.util.List;

import fb.army.entities.Druid;
import fb.army.entities.Entity;
import fb.army.entities.Priest;
import fb.army.entities.SpellCaster;
import fb.army.entities.Wizard;
import fb.artificialintelligence.TargetFinder;
import fb.main.FB;
import fb.main.Battle;
import fb.sound.SoundPlayer;
import fb.spells.Terror;
import fb.spells.BannishDaemon;
import fb.spells.BasicSpell;
import fb.spells.Lightnings;
import fb.spells.EnergyBall;
import fb.spells.Heal;
import fb.spells.MagicShield;
import fb.spells.ProjectileShield;
import fb.spells.Resurection;
import fb.spells.Shrink;
import fb.spells.Speed;
import fb.spells.SummonDaemon;
import fb.spells.Sunrise;
import fb.spells.TargetSpell;
import fb.spells.Telepathy;
import fb.util.DX;
import fb.util.NetworkDX;

/**
 * magic phase for computer players
 * 
 * @author farfpeuf
 * 
 */
public class AIMagicPhase extends BinaryPhase {
	/**
	 * selected spell
	 */
	protected BasicSpell spell;

	/**
	 * current step is spell selection ?
	 */
	private boolean spellSelection = false;

	/**
	 * phase icon number
	 */
	private static final int iconNumber = 5;

	/**
	 * phase name
	 */
	private static final String name = "iamagiqphase";

	/**
	 * @return phase name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return phase icon number
	 */
	public int getIconNumber() {
		return iconNumber;
	}

	/**
	 * @return phase toolip
	 */
	public String getTooltip() {
		return null;// no tooltip
	}

	/**
	 * determine first playing army
	 * 
	 */
	public void initCurrent() {
		if (FB.isNet()) {
			current = NetworkDX.roll(armies.length) - 1;
		} else {
			current = DX.roll(armies.length) - 1;
		}
		Battle.getInstance().setArmy(armies[current]);
		// FB.debug("initCurrent " + armies[current].getPlayerName());
	}

	/**
	 * change current playing army
	 * 
	 */
	public void changeArmy() {
		int i = current + 1;
		if (i == armies.length) {
			i = 0;
		}
		while (Battle.getLoop().getPhaseTokens()[i] && !finished()) {
			i++;
			if (i == armies.length) {
				i = 0;
			}
		}
		current = i;
		// FB.debug("changeArmy " + armies[current].getPlayerName());
	}

	/**
	 * change current playing army and initialize actors selection
	 */
	public void nextArmy() {
		changeArmy();
		if (armies[current].isAi() && !finished()) {
			initActorSelection();
		}
	}

	/**
	 * proceed to phase next step
	 */
	public void nextStep() {

		if (armies[current].isHandled()) {
			if (armies[current].isAi()) {
				handleAI();
			} else {
				setToken(current);
				if (!finished()) {
					nextArmy();
				}
			}
		} else {
			waitNetAction();
		}

	}

	/**
	 * @return AI thread that handles choice of the best target among a set of
	 *         availables targets
	 */
	public TargetFinder getTargetFinder() {
		return new TargetFinder(getPotentialTargets(), 3, 15, getActor()) {
			protected int getWeight(Object element) {
				Entity potentialTarget = (Entity) element;
				int poid = super.getWeight(potentialTarget);
				poid += ((TargetSpell) spell).getWeight(potentialTarget);
				return poid;
			}
		};
	}

	/**
	 * choose spell for computer player
	 * 
	 * @return chosen spell
	 */
	public BasicSpell getAISpell() {
		BasicSpell spell = null;
		int dx;
		if (getActor() instanceof Wizard) {
			dx = DX.roll(6);
			if (dx == 1) {
				spell = new Lightnings();
			} else if (dx == 2) {
				spell = new EnergyBall();
			} else if (dx == 3) {
				spell = new MagicShield();
			} else if (dx == 4) {
				spell = new SummonDaemon();
			} else if (dx == 5) {
				spell = new BannishDaemon();
			} else if (dx == 6) {
				spell = new Telepathy();
			}
		} else if (getActor() instanceof Druid) {
			dx = DX.roll(5);
			if (dx == 1) {
				spell = new Sunrise();
			} else if (dx == 2) {
				spell = new Terror();
			} else if (dx == 3) {
				spell = new Shrink();
			} else if (dx == 4) {
				spell = new Speed();
			} else if (dx == 5) {
				spell = new ProjectileShield();
			}
		} else if (getActor() instanceof Priest) {
			dx = DX.roll(2);
			if (dx == 1) {
				spell = new Resurection();
			} else if (dx == 2) {
				spell = new Heal();
			}
		}
		spell.setCaster((SpellCaster) getActor());
		return spell;
	}

	/**
	 * handle computer player
	 * 
	 */
	public void handleAI() {
		if (!spellSelection) {
			if (isActorSelection()) {
				setActor((Entity) getPotentialActors().get(
						DX.roll(getPotentialActors().size() - 1)));
				spellSelection = true;
				setActorselection(false);
				clearPotentialTargets();
				doneEntities.add(getActor());
			} else {

				setTarget(getAITarget());
				if (getTarget() != null) {
					((TargetSpell) spell).setTarget(getTarget());
					action();
				}

				spell = null;
				initActorSelection();
			}
		} else {
			spell = getAISpell();
			spellSelection = false;
			if (spell.shouldAICast()) {
				if (spell instanceof TargetSpell) {
					calculatePotentialTargets();
				} else {
					action();
				}
			} else {
				spell = null;
				initActorSelection();
			}
		}
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            phase actor
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity actor, Entity e) {
		return true;
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for current actor
	 */
	public boolean isTarget(Entity e) {
		return spell.isTarget(e);
	}

	/**
	 * is this entity a potential actor for this phase
	 * 
	 * @param a
	 *            phase actor
	 * @return true if this entity is a potential actor
	 */
	public boolean isActor(Entity e) {
		if (e.isMagic()) {
			if (((SpellCaster) e).getMagicPoints() > 0) {
				return true;
			}
		}
		return false;
	}

	/**
	 * has this entity targets for this phase ?
	 * 
	 * @param actor
	 *            the phase actor
	 * @return true if this entity has targets for this phase ?
	 */
	public final boolean hasTargets(Entity actor) {
		Entity entity;
		List entities = armies[current].getActiveEntities();
		for (int i = 0; i < entities.size(); i++) {
			entity = (Entity) (entities.get(i));
			if (entity.isActive()) {
				if (isTarget(actor, entity)) {
					return true;
				}
			}
		}
		return false;

	}

	/**
	 * calculate phase results
	 * 
	 */
	public void calculateResults() {
		// nothing to do
	}

	/**
	 * set phase spell
	 * 
	 * @param s
	 *            the spell
	 */
	public synchronized void setSpell(BasicSpell s) {
		// nothing to do
	}

	/**
	 * proceed to phase action
	 * 
	 */
	public void action() {
		SoundPlayer.getInstance().play("magiecast");
		spell.action();
		if (spell instanceof TargetSpell) {
			calculatePotentialTargets();
		}
	}
}