/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.sound;


import java.io.File;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.LineUnavailableException;

import fb.main.FB;
import fb.util.DX;

/**
 * A class that handles a type of sound
 * @author farfpeuf
 *
 */

class LineLoader extends Thread {

	private Clip clip;

	/**
	 * the sounlist in which the clip should be put once loaded.
	 */

	private SoundList soundList;

	/**
	 * the sound files of this sound type.
	 */

	private File[] pathList;

	/**
	 * the type of sound to load.
	 */

	private String soundType;

	/**
	 * Constructs a new line loader for the specified soundlist and with the
	 * specified sound type.
	 */

	public LineLoader(SoundList soundList, String soundType) {

		try {

			// find all the sound files in the directory of this sound type
			// File soundDirectoryPath=new File("mp3/"+soundType);
			File soundDirectoryPath = new File("wav"+File.separator + soundType);
			// if(!soundDirectoryPath.exists()){
			// soundDirectoryPath=new File("mp3/"+soundType);
			// }
			if (soundDirectoryPath.exists()) {
				pathList = soundDirectoryPath.listFiles();
				this.soundList = soundList;
				this.soundType = soundType;
			} else {
				throw new IllegalArgumentException("directory not found wav/"
						+ soundType);
			}
		} catch (Exception e) {
			FB.exception(e);
		}

	}

	/**
	 * start() -> new process to load the clips.
	 */

	public void run() {
		loadclip();
	}

	/**
	 * load a new clip of this sound type.
	 */
	public void loadclip() {

		// choose a random sound file in the sound type directory
		int aleat;
		File audiofile;

		if (pathList.length > 0) {

			aleat = DX.roll(pathList.length);
			audiofile = pathList[aleat - 1];

			try {
				// open an audiostream and get its format
				AudioInputStream stream = AudioSystem
						.getAudioInputStream(audiofile);
				AudioFormat format = stream.getFormat();

				// convert it to PCM_SIGNED if needed

				if (format.getEncoding() != AudioFormat.Encoding.PCM_SIGNED) {
					format = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED,
							format.getSampleRate(), 16, format.getChannels(),
							format.getChannels() * 2, format.getSampleRate(),
							false);
					stream = AudioSystem.getAudioInputStream(format, stream);
				}// if

				// get the clip
				DataLine.Info info = new DataLine.Info(Clip.class, format);
				clip = (Clip) AudioSystem.getLine(info);

				// open the clip (load it)
				try {
					clip.open(stream);
				} catch (LineUnavailableException lue) {
					FB.debug("no more free chanel");
				}

				// add a listener so that when the clip stops the lineloader
				// loads anotherone

				clip.addLineListener(new LineListener() {
					public void update(LineEvent e) {
						if (e.getType().equals(LineEvent.Type.STOP)) {
							clip.close();
							loadclip();
						}
					}
				});

				// add the clip in the soundlist
				soundList.putClip(clip, soundType);

			} catch (Exception e) {
				FB.exception(e);
			}

		}// if
		else {
			FB.debug("no sound files for " + soundType);
		}

	}

}
