/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.spells;

import fb.army.entities.Daemon;
import fb.army.entities.Entity;
import fb.army.entities.SpellCaster;
import fb.util.D6;

/**
 * bannish a summoned daemon, before it drains all the magic power of its
 * summoner.
 * 
 * @author farfpeuf
 * 
 */
public class BannishDaemon extends TargetSpell {

	/**
	 * constructor
	 * 
	 */
	public BannishDaemon() {
		super();
		name = "bannish";
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            spell actor
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity a, Entity e) {
		return e instanceof Daemon
				&& ((Daemon) e).getSummoner() == (SpellCaster) a
				&& a.lineOfSightTo(e) < 9999 && super.isTarget(a, e);
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for current spell
	 *         caster
	 */
	public boolean isTarget(Entity e) {
		return e instanceof Daemon
				&& ((Daemon) e).getSummoner() == (SpellCaster) caster
				&& caster.lineOfSightTo(e) < 9999 && super.isTarget(e);
	}

	/**
	 * cast the spell
	 * 
	 * @return true if the spell was successfull
	 */
	public boolean castSpell() {

		if (caster.spendMagicPoints(cost())) {
			return true;
		}

		return false;

	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell cost
	 */
	public int cost(SpellCaster caster, Entity target) {
		return D6.roll() + D6.roll() + (int) (caster.stepsTo(target) / 3);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell maximum cost
	 */
	public int maxCost(SpellCaster caster, Entity target) {
		return 12 + (int) (caster.stepsTo(target) / 3);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell minimum cost
	 */
	public int minCost(SpellCaster caster, Entity target) {
		return 2 + (int) (caster.stepsTo(target) / 3);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return average cost
	 */
	public int averageCost(SpellCaster caster, Entity target) {
		return 7 + (int) (caster.stepsTo(target) / 3);
	}

	/**
	 * end the spell
	 * 
	 * @throws InterruptedException
	 */
	public void endSpell() throws InterruptedException {

		((Daemon) target).doBannish();
		magicSynchronize();

	}

	/**
	 * @param potentialTarget
	 *            the target
	 * @return weight modifiers for artificial intelligence for specified target
	 */
	public int getWeight(Entity potentialTarget) {
		int weight = 0;
		if (caster.getMagicPoints() < 5) {
			weight += 2;
		} else if (caster.getMagicPoints() < 10) {
			weight += 4;
		} else {
			weight -= 2;

		}
		if (potentialTarget.stepsTo(caster) < potentialTarget.getMovement()) {
			weight += 2;
		}

		return weight;
	}

}
