/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.spells;

import java.util.Iterator;

import fb.army.entities.Entity;
import fb.army.entities.Unit;
import fb.main.Battle;
import fb.util.D6;
import fb.util.Resources;

/**
 * Enables to go forward into time
 * 
 * @author farfpeuf
 * 
 */
public class Sunrise extends BasicSpell implements ValuedSpell {
	/**
	 * added time slots
	 */
	private int timePlus = 1;

	public Sunrise() {
		super();
		name = "sunrise";
	}

	/**
	 * @return string to send spell to remote computers
	 */
	protected String spellString() {
		return super.spellString() + "#" + currentValue();
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            spell actor
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity a, Entity e) {
		return false;
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for current spell
	 *         caster
	 */
	public boolean isTarget(Entity e) {
		return false;
	}

	/**
	 * cast the spell
	 * 
	 * @return true if the spell was successfull
	 */
	public boolean castSpell() {
		if (caster.spendMagicPoints(cost())) {
			return true;
		}
		return false;
	}

	/**
	 * @return spell cost
	 */
	public int cost() {
		int cost = 0;
		for (int i = 0; i < java.lang.Math.abs(timePlus); i++) {
			cost += (D6.roll() + D6.roll() + D6.roll());
		}
		return cost;
	}

	/**
	 * @return spell maximum cost
	 */
	public int maxCost() {
		return java.lang.Math.abs(timePlus) * 18;
	}

	/**
	 * @return spell minimum cost
	 */
	public int minCost() {
		return java.lang.Math.abs(timePlus) * 3;
	}

	/**
	 * @return spell average cost
	 */
	public int averageCost() {
		return (int) (java.lang.Math.abs(timePlus) * 10.5);
	}

	/**
	 * end the spell
	 * 
	 * @throws InterruptedException
	 */
	public void endSpell() throws InterruptedException {
		Battle.getInstance().timePlus(timePlus);
		magicSynchronize();
	}

	/**
	 * @return current spell value
	 */
	public int currentValue() {
		return timePlus;
	}

	/**
	 * @return maximum value
	 */
	public int maxValue() {
		return 9;
	}

	/**
	 * @return minimum value
	 */
	public int minValue() {
		return 1;
	}

	/**
	 * @return the value name
	 */
	public String valueName() {
		return Resources.getString("sunriseValue");
	}

	/**
	 * set the value
	 * 
	 * @param value
	 *            the new value
	 */
	public void setValue(int value) {
		timePlus = value;
	}

	/**
	 * @return modifiers for artificial intelligence
	 */
	public int getWeightModifiers() {
		int result = super.getWeightModifiers() - 10;
		Iterator iter = caster.getArmy().getActiveEntities().iterator();
		Entity entity;
		while (iter.hasNext()) {
			entity = (Entity) iter.next();
			if (entity instanceof Unit) {
				if (entity.suffersBadLight()) {
					result += 3;
				} else if (entity.getBadLight() != '0') {
					result -= 3;
				}
			}
		}
		return result;
	}
}