/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.util;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.net.URL;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import javax.swing.ImageIcon;
import fb.main.FB;

/**
 * utility class to read from and write to encrypted and non encrypted files
 * 
 * @author Farfpeuf
 */
public class FileTools {
	// -----------write to file------------------
	/**
	 * write to a file (no encryption)
	 * 
	 * @param path
	 *            the file path
	 * @param stream
	 *            the string to write
	 */
	public static void writeFile(String path, String stream) {
		writeFile(path, stream, false);
	}

	/**
	 * write to a file (no encryption)
	 * 
	 * @param file
	 *            the file
	 * @param stream
	 *            the string to write
	 */
	public static void writeFile(File file, String stream) {
		writeFile(file, stream, false);
	}

	/**
	 * write to a file (no encryption)
	 * 
	 * @param output_stream
	 *            the file output stream
	 * @param stream
	 *            the string to write
	 */
	public static void writeFile(OutputStream output_stream, String stream)
			throws Exception {
		writeFile(output_stream, stream, false);
	}

	/**
	 * write to a file
	 * 
	 * @param path
	 *            the file path
	 * @param stream
	 *            the string to write
	 * @param encrypted
	 *            encrypt file ?
	 */
	public static void writeFile(String path, String stream, boolean encrypted) {
		try {
			if (encrypted) {
				path = getEncodedPath(path);
			}
			writeFile(new FileOutputStream(path), stream, encrypted);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * write to a file
	 * 
	 * @param file
	 *            the file
	 * @param stream
	 *            the string to write
	 * @param encrypted
	 *            encrypt file ?
	 */
	public static void writeFile(File file, String stream, boolean encrypted) {
		try {
			writeFile(new FileOutputStream(file), stream, encrypted);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * write to a file
	 * 
	 * @param output_stream
	 *            the file output stream
	 * @param stream
	 *            the string to write
	 * @param encrypted
	 *            encrypt file ?
	 */
	public static void writeFile(OutputStream output_stream, String stream,
			boolean encrypted) throws Exception {
		if (encrypted) {
			ByteArrayInputStream bais = new ByteArrayInputStream(stream
					.getBytes());
			Crypto.getInstance().encrypt(bais, output_stream);
			bais.close();
		} else {
			OutputStreamWriter osw = new OutputStreamWriter(output_stream);
			BufferedWriter bw = new BufferedWriter(osw);
			bw.write(stream);
			bw.close();
			osw.close();
			output_stream.close();
		}
	}

	// -----------read from file---------------------------
	/**
	 * read a file content (not encrypted)
	 * 
	 * @param path
	 *            the file path
	 * @return the file content as a string
	 */
	public static String readFile(String path) {
		return readFile(path, false);
	}

	/**
	 * read a file content (not encrypted)
	 * 
	 * @param input_stream
	 *            the file input stream
	 * @return the file content as a string
	 */
	public static String readFile(InputStream input_stream) throws Exception {
		return readFile(input_stream, false);
	}

	/**
	 * read a file content
	 * 
	 * @param path
	 *            the file path
	 * @param encrypted
	 *            encrypted file ?
	 * @return the file content as a string
	 */
	public static String readFile(String path, boolean encrypted) {
		if (encrypted) {
			path = getEncodedPath(path);
		}
		try {
			return readFile(getRessourceStream(path), encrypted);
		} catch (Exception e) {
			FB.exception(e);
		}
		return null;
	}

	/**
	 * read a file content
	 * 
	 * @param input_stream
	 *            the file input stream
	 * @param encrypted
	 *            encrypted file ?
	 * @return the file content as a string
	 */
	public static String readFile(InputStream input_stream, boolean encrypted)
			throws Exception {
		String filestring = null;
		if (encrypted) {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			Crypto.getInstance().decrypt(input_stream, baos);
			filestring = new String(baos.toByteArray());
		} else {
			byte[] byteArray = new byte[input_stream.available()];
			input_stream.read(byteArray);
			filestring = new String(byteArray);
		}
		input_stream.close();
		return filestring;
	}

	/**
	 * get a file reader (not encrypted)
	 * 
	 * @param path
	 *            the file path
	 * @return the file reader
	 */
	public static Reader getReader(String path) {
		return getReader(path, false);
	}

	/**
	 * get a file reader
	 * 
	 * @param path
	 *            the file path
	 * @param encrypted
	 *            encrypted file ?
	 * @return the file reader
	 */
	public static Reader getReader(String path, boolean encrypted) {
		try {
			if (encrypted) {
				path = getEncodedPath(path);
			}
			return getReader(getRessourceStream(path), encrypted);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * get a file reader
	 * 
	 * @param input_stream
	 *            the file input stream
	 * @return the file reader
	 */
	public static Reader getReader(InputStream input_stream) throws Exception {
		return getReader(input_stream, false);
	}

	/**
	 * get a file reader
	 * 
	 * @param input_stream
	 *            the file input stream
	 * @param encrypted
	 *            encrypted file ?
	 * @return the file reader
	 */
	public static Reader getReader(InputStream input_stream, boolean encrypted)
			throws Exception {
		Reader reader = null;
		if (encrypted) {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			Crypto.getInstance().decrypt(input_stream, baos);
			reader = new InputStreamReader(new ByteArrayInputStream(baos
					.toByteArray()));
		} else {
			reader = new InputStreamReader(input_stream);
		}
		return reader;
	}

	/**
	 * get an icon from a file (not encrypted)
	 * 
	 * @param path
	 *            the file path
	 * @return the icon
	 */
	public static ImageIcon getIcon(String path) {
		return getIcon(path, false);
	}

	/**
	 * get an icon from a file
	 * 
	 * @param path
	 *            the file path
	 * @param encrypted
	 *            encrypted file ?
	 * @return the icon
	 */
	public static ImageIcon getIcon(String path, boolean encrypted) {
		InputStream stream = null;
		try {
			if (encrypted) {
				path = getEncodedPath(path);
			}
			stream = getRessourceStream(path);
			if (stream == null) {
				FB.debug("image not found " + path);
				return new ImageIcon();
			}
			ByteArrayOutputStream baos = new ByteArrayOutputStream();

			if (encrypted) {
				Crypto.getInstance().decrypt(stream, baos);
				return new ImageIcon(baos.toByteArray());
			} else {
				byte[] buf = new byte[1024];
				int numRead = 0;
				while ((numRead = stream.read(buf)) >= 0) {
					baos.write(buf, 0, numRead);
				}
				Image image = Toolkit.getDefaultToolkit().createImage(
						baos.toByteArray());
				stream.close();
				baos.close();
				return new ImageIcon(image);

			}
		} catch (Exception e) {
			FB.exception(e);
			return new ImageIcon();
		}
	}

	/**
	 * get a file encoded path
	 * 
	 * @param path
	 *            the file path
	 * @return the file encoded path
	 */
	public static String getEncodedPath(String path) {
		int index = path.lastIndexOf('.');
		if (index == -1) {
			return path;
		}
		return path.substring(0, index);
	}

	/**
	 * Tests whether the file (not encrypted) or directory denoted by this
	 * abstract pathname exists.
	 * 
	 * @param path
	 *            the file path
	 * @return true if and only if the file or directory denoted by this
	 *         abstract pathname exists
	 * 
	 */
	public static boolean exists(String path) {
		return exists(path, false);
	}

	/**
	 * Tests whether the file or directory denoted by this abstract pathname
	 * exists.
	 * 
	 * @param path
	 *            the file path
	 * @param encrypted
	 *            encrypted file ?
	 * @return true if and only if the file or directory denoted by this
	 *         abstract pathname exists
	 * 
	 */
	public static boolean exists(String path, boolean encrypted) {
		if (encrypted) {
			path = getEncodedPath(path);
		}
		InputStream stream = ClassLoader.getSystemResourceAsStream(path
				.replace(File.separatorChar, '/'));
		return (stream != null);
	}

	/**
	 * list the files in a directory
	 * 
	 * @param dir
	 *            the directory paht
	 * @return the files list
	 */
	public static List list(String dir) {
		try {
			URL url = ClassLoader.getSystemResource(dir);
			String path = url.getPath();
			File file = new File(URLDecoder.decode(path, "UTF-8"));
			if (file.exists()) {
				return Arrays.asList(file.list());
			}
			if (url.getProtocol().equals("jar")) {
				String jarPath = path.substring(5, path.lastIndexOf('!'));
				jarPath = URLDecoder.decode(jarPath, "UTF-8");
				JarFile jarFile = new JarFile(jarPath);
				List files = new ArrayList();
				Enumeration e = jarFile.entries();
				JarEntry jarEntry;
				while (e.hasMoreElements()) {
					jarEntry = (JarEntry) e.nextElement();
					if (!jarEntry.isDirectory()
							&& jarEntry.getName().startsWith(dir)) {
						files.add(jarEntry.getName()
								.substring(dir.length() + 1));
					}
				}
				jarFile.close();
				return files;
			}
		} catch (IOException e) {
			FB.exception(e);
		}
		return null;
	}

	/**
	 * test if file is a directory
	 * 
	 * @param dir
	 *            the file path
	 * @return true if the file is a directory
	 */
	public static boolean isDirectory(String dir) {
		try {
			URL url = ClassLoader.getSystemResource(dir);
			String path = url.getPath();
			File file = new File(URLDecoder.decode(path, "UTF-8"));
			if (file.exists()) {
				return file.isDirectory();
			}
			if (url.getProtocol().equals("jar")) {
				String jarPath = path.substring(5, path.lastIndexOf('!'));
				jarPath = URLDecoder.decode(jarPath, "UTF-8");
				JarFile jarFile = new JarFile(jarPath);
				boolean isDir = (jarFile.getEntry(dir + "/") != null);
				jarFile.close();
				return isDir;
			}
		} catch (IOException e) {
			FB.exception(e);
		}
		return false;
	}

	/**
	 * get a ressource as an input stream
	 * 
	 * @param path
	 *            the resource path
	 * @return the input stream
	 * @throws Exception
	 */
	public static InputStream getRessourceStream(String path) throws Exception {
		path = path.replace(File.separatorChar, '/');
		InputStream stream = ClassLoader.getSystemResourceAsStream(path);
		if (stream == null) {
			FB.debug("resource not found " + path);
		}
		return stream;
	}
}
